// SVXConverterDlg.cpp : implementation file
//
// CONTAINS THE MAIN CONVERSION CODE

// AUTHOR: ROMAN VETTER
// DATE: JUNE 5, 2011
// VERSION: 1.07
// CONTACT: WWW.TT-MS.DE/SVXCONVERTER

#include "stdafx.h"
#include "SVXConverter.h"
#include "SVXConverterDlg.h"
#include "SVXConverterSubWin.h"
#include <string>
#include <stdio.h>

using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// functions used for surplus town/industry removal

// sorts an array of strings lexicographically and returns a dynamic array
// recording the rearrangement (used for sorting town/industry lists)
int * bubbleSort(char ** array, int size)
{
	int * order = new int [size];
	for (int i = 0; i < size; ++i) order[i] = i; // 0,1,2,3,4...

	int bound = size-1;
	while (bound > 0)
	{
		int lastswap = 0;
		for (int pos = 0; pos < bound; ++pos)
		{
			if (string(array[pos]) > string(array[pos+1]))
			{
				// swap strings (char pointers)
				char * ctemp = array[pos];
				array[pos] = array[pos+1];
				array[pos+1] = ctemp;
				
				// swap order entries
				int itemp = order[pos];
				order[pos] = order[pos+1];
				order[pos+1] = itemp;
				
				lastswap = pos;
			}
		}
		bound = lastswap;
	}

	return order;
}

// generates a town name from its parts
char * getTownName(unsigned char lang, unsigned long parts)
{
	// german town name parts
	const char * german1 [23] = {
		"Bruns","Lim","Han","Brun","Ober","Frank","Rott","Frei","Ravens",
		"Schwein","Osna","Dssel","Wester","Flens","Mhl","Heidel",
		"Inns","Cloppen","Pfung","Michel","Wert","Wildes","Freuden"};
	const char * german2 [14] = {
		"gen","stadt","heim","burg","haven","ford","feld","mund",
		"mnster","hausen","furt","brck","over","wald"};

	// english/american town name parts
	const char * english1 [4] = {"Great ","Little ","New ","Fort "};
	const char * english2 [26] = {
		"Wr","B","C","Ch","Br","D","Dr","F","Fr","Fl","G","Gr","H",
		"L","M","N","P","Pr","Pl","R","S","S","Sl","T","Tr","W"};
	const char * english3 [8] = {"ar","a","e","in","on","u","un","en"};
	const char * english4 [7] = {"n","ning","ding","d","","t","fing"};
	const char * english5 [23] = {
		"ville","ham","field","ton","town","bridge","bury","wood",
		"ford","hall","ston","way","stone","borough","ley","head",
		"bourne","pool","worth","hill","well","hattan","burg"};
	const char * english6 [9] = {
		"-on-sea"," Bay"," Market"," Cross"," Bridge"," Falls"," City"," Ridge"," Springs"};

	string namestr = "";
	char * name = new char [48]; // maximum length of an industry name is 47 characters
	for (int i = 0; i < 48; ++i) name[i] = 0;
	
	if (lang == 1) // german town name (consists of two mandatory parts only)
	{
		// assemble town name
		namestr  = german1[(( parts       & 0xFFFF) * 23) >> 16];
		namestr += german2[(((parts >> 5) & 0xFFFF) * 14) >> 16];
		memcpy(name, namestr.c_str(), namestr.length());
	}
	else // english or american
	{
		// assemble town name (6 parts, first and last optional)
		int t = (((parts & 0xFFFF) * 54) >> 16) - 50;
		if (t >= 0) namestr = english1[t];
		int firstpartlength = namestr.length(); // used for unwanted words replacement
		namestr += english2[(((parts >>  4) & 0xFFFF) * 26) >> 16];
		namestr += english3[(((parts >>  7) & 0xFFFF) *  8) >> 16];
		namestr += english4[(((parts >> 10) & 0xFFFF) *  7) >> 16];
		namestr += english5[(((parts >> 13) & 0xFFFF) * 23) >> 16];
		t = ((((parts >> 15) & 0xFFFF) * 69) >> 16) - 60;
		if (t >= 0) namestr += english6[t];

		memcpy(name, namestr.c_str(), namestr.length());

		// replace unwanted words
		char * start = name + firstpartlength; // beginning of the main part of the name
		if (start[0] == 'C' && (start[1] == 'e' || start[1] == 'i')) start[0] = 'K';
		if (strncmp(start, "Cunt", 4) == 0) strncpy(start, "East", 4);
		if (strncmp(start, "Slag", 4) == 0) strncpy(start, "Pits", 4);
		if (strncmp(start, "Slut", 4) == 0) strncpy(start, "Edin", 4);
		if (strncmp(start, "Fart", 4) == 0) strncpy(start, "Boot", 4);
		if (strncmp(start, "Drar", 4) == 0) strncpy(start, "Quar", 4);
		if (strncmp(start, "Dreh", 4) == 0) strncpy(start, "Bash", 4);
		if (strncmp(start, "Frar", 4) == 0) strncpy(start, "Shor", 4);
		if (strncmp(start, "Grar", 4) == 0) strncpy(start, "Aber", 4);
		if (strncmp(start, "Brar", 4) == 0) strncpy(start, "Over", 4);
		if (strncmp(start, "Wrar", 4) == 0) strncpy(start, "Inve", 4);
	}
	
	return name;
}

void removeIndustry(unsigned char ind, unsigned char * const tto_dec)
{
	unsigned char * ptr = &tto_dec[0x29E16 + ind*0x28]; // pointer to the industry

	// level the tiles taken by the industry
	unsigned short xy;
	memcpy(&xy, ptr, 2);
	unsigned char dimx = *(ptr + 6);
	unsigned char dimy = *(ptr + 7);
	for (unsigned char i = 0; i < dimy; ++i)
	{
		for (unsigned char j = 0; j < dimx; ++j)
		{
			if (tto_dec[0x4971D + xy] / 16 == 8) // tile class 8: industry
			{
				if (tto_dec[0x13BA6 + xy] == ind) // belongs to this very industry
				{
					if (*(ptr + 0x24) == 5) // oil rig
					{
						tto_dec[0x03BA6 + xy] = 0x11; // L1: owner
						tto_dec[0x4971D + xy] &= 0xF; // L4:
						tto_dec[0x4971D + xy] |= 0x60;// class 6 (upper 4 bits)
						tto_dec[0x5971D + xy] = 0;	  // L5: water
					}
					else
					{
						tto_dec[0x03BA6 + xy] = 0x10; // L1: owner
						tto_dec[0x4971D + xy] &= 0xF; // L4: class 0 (upper 4 bits)
						tto_dec[0x5971D + xy] = 0;	  // L5: bare land, update counter = 0
					}
				}
			}
			++xy;
		}
		xy += 0x100 - dimx;
	}

	// mark industry array slot as empty
	memset(ptr, 0, 2);
	
	// delete subsidies with this industry
	for (ptr = &tto_dec[0x496AE]; ptr != &tto_dec[0x496CE]; ptr += 4)
	{
		if (*ptr != 0xFF && *ptr != 0 && *ptr != 2) // active entry && neither passengers nor mail
			if (*(ptr+2) == ind || *ptr != 5 && *(ptr+3) == ind) // source = ind || cargo != goods && dest = ind
				*ptr = 0xFF; // deactivate subsidy
	}
}

void removeTown(unsigned char town, unsigned char * const tto_dec)
{
	unsigned char * ptr = &tto_dec[0x264 + town*0x4E]; // pointer to the town

	// remove the town from the alphabetically sorted list
	unsigned char pos; // position in the list
	memcpy(&pos, ptr + 11, 1);
	for (unsigned long tto_off = 0x264; tto_off != 0x1AC4; tto_off += 0x4E) // iterate over all towns
	{
		if (tto_dec[tto_off] || tto_dec[tto_off + 1]) // if active town entry
			if (pos < tto_dec[tto_off + 11]) // if alphabetical position further behind
				--tto_dec[tto_off + 11]; // move it forward by 1
	}

	// create an array storing which town has what industries
	unsigned char * towninds = new unsigned char[80*12] ; // 80 towns with at most 12 industries each
	for (int i = 0; i < 80*12; ++i) towninds[i] = 255;	  // initial value: industry not present
	unsigned char * indptr; // pointer to an entry in the industry array
	unsigned long pointer;  // "pointer" to the town entry the industry belongs to
	unsigned char t;		// town array index
	for (int i = 0; i < 100; ++i) // iterate over all industries
	{
		indptr = &tto_dec[0x29E16 + i*0x28];
		if (*indptr || *(indptr+1)) // active industry entry
		{
			memcpy(&pointer, indptr + 2, 4);
			t = (pointer - 0x264) / 0x4E; // convert the "pointer" to a town array index
			towninds[t*12 + *(indptr + 0x24)] = i; // store index at ind.type-corresponding position
		}
	}

	// try to assign the industries assigned to this town to others or remove them
	short tx, ty, ix, iy;	   // town and industry coordinates
	unsigned short dist, best; // pseudo-distances between industry & town
	unsigned char closest;	   // index of the closest town
	unsigned char ind;		   // industry array index
	unsigned char * townptr;   // pointer to an entry in the town array
	for (int i = 0; i < 12; ++i)   // iterate over all industry types
	{
		// find the pseudo-closest free town
		ind = towninds[town*12+i]; // index of the industry we want to reassign
		if (ind == 255) continue;  // no industry of type i at this town
		indptr = &tto_dec[0x29E16 + ind*0x28];
		ix = *indptr;     // x position of the industry to be reassigned
		iy = *(indptr+1); // y
		best = (unsigned short)-1;
		for (t = 0; t < 80; ++t) // iterate over all towns
		{
			townptr = &tto_dec[0x264 + t*0x4E];
			tx = *townptr;	   // x position of the candidate town
			ty = *(townptr+1); // y
			if (tx || ty) // if active town entry
			{
				if (towninds[t*12+i] == 255) // if the industry slot for this type is free
				{
					dist = abs(ix-tx) + abs(iy-ty); // pseudo-distance calculation like in TTD
					if (dist < best)
					{
						closest = t; // this one is closer
						best = dist;
					}
				}
			}
		}

		// reassign the industry to that town or remove it
		if (best == (unsigned short)-1) // no free town found
		{
			removeIndustry(ind, tto_dec);
		}
		else
		{
			pointer = 0x264 + closest*0x4E;  // convert the town array index to a "pointer"
			memcpy(indptr + 2, &pointer, 4); // reassign the industry
		}
	}
	delete [] towninds;

	// reassign townfolk to other towns
	unsigned char inhabitants [36] = {
		187, 85, 40,  5,220, 30,140,  0,  0,  0,  0,  0,150, 95, 95, 95,130,  6,
		110, 65,  0,  0,  0, 15, 12, 13,100,170,100,180, 35, 65,  0,  0,  0,140};
	unsigned short population; // town population
	unsigned short secbest;    // pseudo-distance between a tile & 2nd closest town
	unsigned char secclosest;  // index of the 2nd closest town
	unsigned char tileclass;
	for (int i = 0; i < 0x10000; ++i)
	{
		tileclass = tto_dec[0x4971D + i] / 16;
		if (tileclass == 3) // if town building
		{
			// find the pseudo-closest towns
			ix = i % 0x100;
			iy = i / 0x100;
			best = secbest = (unsigned short)-1;
			for (t = 0; t < 80; ++t) // iterate over all towns
			{
				townptr = &tto_dec[0x264 + t*0x4E];
				tx = *townptr;	   // x position of the candidate town
				ty = *(townptr+1); // y
				if (tx || ty) // if active town entry
				{
					dist = abs(ix-tx) + abs(iy-ty); // pseudo-distance calculation like in TTD
					if (dist < best)
					{
						secclosest = closest;
						closest = t;
						secbest = best;
						best = dist;
					}
					else if (dist < secbest)
					{
						secclosest = t;
						secbest = dist;
					}
				}
			}

			// move townsfolk to it if building is fully constructed yet and belongs to 'town'
			if (closest == town && (tto_dec[0x13BA6 + i] & 192) == 192)
			{
				memcpy(&population, tto_dec + 0x266 + secclosest*0x4E, 2);
				population += inhabitants[tto_dec[0x13BA6 + i] & 63];
				memcpy(tto_dec + 0x266 + secclosest*0x4E, &population, 2);
			}
		}
	}

	// mark town array slot as empty
	memset(ptr, 0, 2);

	// reassign tiles to other towns
	for (int i = 0; i < 0x10000; ++i)
	{
		tileclass = tto_dec[0x4971D + i] / 16;
		if (tileclass != 3 && tileclass != 7 && tileclass != 8 && tileclass < 11
			&& tto_dec[0x3BA6 + i] == 128 + town) // if 'town' owns this tile
		{
			// find the pseudo-closest town
			ix = i % 0x100;
			iy = i / 0x100;
			best = (unsigned short)-1;
			for (t = 0; t < 80; ++t) // iterate over all towns
			{
				townptr = &tto_dec[0x264 + t*0x4E];
				tx = *townptr;	   // x position of the candidate town
				ty = *(townptr+1); // y
				if (tx || ty) // if active town entry
				{
					dist = abs(ix-tx) + abs(iy-ty); // pseudo-distance calculation like in TTD
					if (dist < best)
					{
						closest = t; // this one is closer
						best = dist;
					}
				}
			}

			// make it the new owner
			tto_dec[0x3BA6 + i] = 128 + closest;
		}
	}

	// delete the custom town name if necessary
	unsigned short text_id;
	memcpy(&text_id, ptr + 4, 2); // get the name string id
	if (text_id != 0x10B6) // if town name not auto-generated
		memset(&tto_dec[0x45746 + 24*(text_id & 0x1FF)], 0, 24); // delete its custom string

	// delete subsidies with this town
	for (ptr = &tto_dec[0x496AE]; ptr != &tto_dec[0x496CE]; ptr += 4)
	{
		if (*ptr == 0 || *ptr == 2 || *ptr == 5) // either passengers, mail or goods
			if (*ptr != 5 && *(ptr+2) == town || *(ptr+3) == town) // cargo != goods && source = town || dest = town
				*ptr = 0xFF; // deactivate subsidy
	}
}

/////////////////////////////////////////////////////////////////////////////
// CSVXConverterDlg dialog

CSVXConverterDlg::CSVXConverterDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSVXConverterDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSVXConverterDlg)
	m_tto = _T("");
	m_ttd = _T("00");
	m_doing = _T("Ready");
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CSVXConverterDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSVXConverterDlg)
	DDX_Control(pDX, IDC_PROGRESSLINE, m_progressline);
	DDX_Text(pDX, IDC_TTO, m_tto);
	DDX_Text(pDX, IDC_TTD, m_ttd);
	DDV_MaxChars(pDX, m_ttd, 2);
	DDX_Text(pDX, IDC_DOING, m_doing);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CSVXConverterDlg, CDialog)
	//{{AFX_MSG_MAP(CSVXConverterDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_ABOUT, OnAbout)
	ON_EN_CHANGE(IDC_TTO, OnChangeTto)
	ON_EN_CHANGE(IDC_TTD, OnChangeTtd)
	ON_BN_CLICKED(IDC_BROWSE, OnBrowse)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSVXConverterDlg message handlers

BOOL CSVXConverterDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog. The framework does this automatically
	// when the application's main window is not a dialog.
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	return TRUE;  // return TRUE unless you set the focus to a control
}

void CSVXConverterDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
// the minimized window.
HCURSOR CSVXConverterDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CSVXConverterDlg::OnAbout()
{
	MessageBox("Copyright  by Roman Vetter\n\nVersion 1.07 (June 5, 2011)\n\nSVXConverter is a tool that converts Transport\nTycoon Original savegames to the TT Deluxe format.\n\nFeel free to contact me at roman@tt-ms.de.\n\nFor further information, please visit\nhttp://www.tt-ms.de/svxconverter", "About SVXConverter");
}

void CSVXConverterDlg::OnCancel() 
{
	CDialog::OnCancel();
}

void CSVXConverterDlg::OnChangeTto() 
{
	UpdateData(TRUE);
	if (m_tto == "")
		{
			GetDlgItem(IDOK)->EnableWindow(FALSE);
		}
		else
		{
			GetDlgItem(IDOK)->EnableWindow(TRUE);
		}
	GetDlgItem(IDC_PROGRESS)->EnableWindow(FALSE);
	GetDlgItem(IDC_PROGRESSLINE)->EnableWindow(FALSE);
	m_doing = "Ready";
	m_progressline.SetPos(0); // reset progress bar
	UpdateData(FALSE);
	GetDlgItem(IDC_DOING)->EnableWindow(FALSE);
}

void CSVXConverterDlg::OnChangeTtd() 
{
	GetDlgItem(IDC_PROGRESS)->EnableWindow(FALSE);
	GetDlgItem(IDC_PROGRESSLINE)->EnableWindow(FALSE);
	GetDlgItem(IDC_DOING)->EnableWindow(FALSE);
	UpdateData(TRUE);
	m_progressline.SetPos(0); // reset progress bar
	m_doing = "Ready";
	UpdateData(FALSE);
	if (strlen(m_ttd) < 2)
		{
			GetDlgItem(IDOK)->EnableWindow(FALSE);
		}
		else if (m_tto != "")
		{
			GetDlgItem(IDOK)->EnableWindow(TRUE);
		}
}

void CSVXConverterDlg::OnBrowse() 
{
	CFileDialog m_selectTTO(TRUE);
	if (m_selectTTO.DoModal() == IDOK)
	{
		m_tto = m_selectTTO.GetPathName();
		UpdateData(FALSE);
		if (m_tto == "")
		{
			GetDlgItem(IDOK)->EnableWindow(FALSE);
		}
		else if (strlen(m_ttd) == 2)
		{
			GetDlgItem(IDOK)->EnableWindow(TRUE);
		}
		GetDlgItem(IDC_PROGRESS)->EnableWindow(FALSE);
		GetDlgItem(IDC_PROGRESSLINE)->EnableWindow(FALSE);
		m_progressline.SetPos(0); // reset progress bar
		m_doing = "Ready";
		UpdateData(FALSE);
		GetDlgItem(IDC_DOING)->EnableWindow(FALSE);
	}
}

void CSVXConverterDlg::OnOK() 
{
	//////////////////////////////////////////////////////////
	// The conversion procedure contains three basic steps: //
	// 1. decompression of the TTO savegame, because all TT //
	//    savegames are run length encoded (RLE)            //
	// 2. reformation and adjustment of all data to make it //
	//    fit the TTD format, and adding TTD-only data      //
	// 3. recompression of the TTD savegame                 //
	//////////////////////////////////////////////////////////

	int bspos = m_tto.ReverseFind('\\') + 1;			// search for last backslash
	string ttdpath = m_tto.Left(bspos);						// determine TTD path (same as TTO path)
	string ttofile = m_tto.Right(m_tto.GetLength() - bspos); // determine TTO file name
	string ttdfile = "TRT" + m_ttd + ".SV" + m_tto.Right(1); // create TTD file name

	// foolproofing: TTO input file does not exist
	FILE * tto = fopen(m_tto, "rb"); // open TTO savegame (read in binary mode)
	if (tto == NULL)
	{
		MessageBox("Please correct the TTO file path.\nThat file doesn't exist.", "Incorrect TTO file path");
		return;
	}

	// foolproofing: TTO input file is not a TTO savegame
	fseek(tto, 0, SEEK_END);
	unsigned long tto_size = ftell(tto); // input file size
	fseek(tto, 0, SEEK_SET);
	unsigned char tto_title [39];	// TTO savegame title
	unsigned char title_checksum [2]; // title checksum
	if (tto_size >= 47) // input file big enough?
	{
		// if title checksum bytes are correct, assume file is a TTO savegame
		fread(tto_title, 1, 39, tto);	// read savegame title
		fread(title_checksum, 1, 2, tto); // read title checksum
		unsigned short tcsum = 0;	// calculated title checksum
		for(int i = 0; i < 39; ++i) // calculate title checksum as it should be
		{
			tcsum += tto_title[i];
			tcsum = (unsigned short)((tcsum << 1) | (tcsum >> 15));
		}
		tcsum ^= 0xAAAA;
		if (tcsum != 0x100 * title_checksum[1] + title_checksum[0]) // calculated != actual checksum?
		{
			fclose(tto);
			MessageBox("The selected input file is not a TTO savegame.", "No TTO savegame");
			return;
		}
	}
	else // input file too small
	{
		fclose(tto);
		MessageBox("The selected input file is not a TTO savegame.", "No TTO savegame");
		return;
	}

	// foolproofing: illegal TTD output file name
	if (m_ttd[0] < '0' || m_ttd[1] < '0' || m_ttd[0] > '9' || m_ttd[1] > '9')
	{
		fclose(tto);
		MessageBox("Please correct the TTD file name.\nOnly numbers are permitted.", "Incorrect TTD file name");
		return;
	}

	// foolproofing: TTD output file already exists
	FILE * existing_ttd = fopen((ttdpath + ttdfile).c_str(), "rb");
	if (existing_ttd != NULL)
	{
		fclose(existing_ttd);
		if (MessageBox(("Do you really want to overwrite the\nexisting savegame (" + ttdfile + ")?").c_str(), "TTD savegame file already exists", MB_OKCANCEL) == IDCANCEL)
			return;
	}

	GetDlgItem(IDC_PROGRESS)->EnableWindow(TRUE);
	GetDlgItem(IDC_PROGRESSLINE)->EnableWindow(TRUE);
	GetDlgItem(IDC_DOING)->EnableWindow(TRUE);

	m_progressline.SetRange(0, 100); // define progress bar range
	m_doing = ("Decompressing " + ttofile).c_str(); // refresh state information
	UpdateData(FALSE);

	//////////////////////////////////////////////////////////
	// 1. TTO savegame decompression
	//
	// TTO savegames are built up as follows:
	// * savegame title (39 bytes, not encoded)
	// * title checksum (2 bytes, not encoded)
	// * saved data (about 20 - 400 KB, run length encoded)
	// * savegame checksum (4 bytes, not encoded)

	// read compressed data to memory (file checksum isn't needed)
	fseek(tto, 41, SEEK_SET);		  // set reading pointer to beginning
	tto_size -= 45;					  // size of RLE data (file size minus title and checksums)
	unsigned char * tto_co = new unsigned char [tto_size]; // compressed data
	fread(tto_co, 1, tto_size, tto);  // read compressed data to memory
	fclose(tto);					  // TTO file no longer needed

	// prepare for TTO decompression
	const unsigned char *ptto_co = tto_co; // pointer to compressed data
	const unsigned char *const ptto_co_end = tto_co + tto_size; // pointer past the end of RLE data
	unsigned char * tto_dec = new unsigned char [431901]; // decompressed TTO data (431901 bytes)
	signed char rle_c; // RLE byte counter
	unsigned char rb;	// repeated byte (temporary)
	unsigned long dec_c = 0; // tto_dec writing counter

	// RLE decompression (each iteration decodes one RLE sequence)
	while (ptto_co < ptto_co_end)
	{
		rle_c = *ptto_co; // read byte count into 'rle_c'
		++ptto_co;
		if (rle_c < 0)
		{	// repeat the byte
			rb = *ptto_co; // this is the byte to repeat
			++ptto_co;
			do
			{
				tto_dec[dec_c] = rb; // write the byte to tto_dec
				++dec_c;
			}
			while(rle_c++); // do it (-rle_c+1) times
		}
		else
		{	// copy the adjacent bytes
			do
			{
				tto_dec[dec_c] = *ptto_co; // write the next (rle_c+1) bytes to tto_dec
				++ptto_co;
				++dec_c;
			}
			while(rle_c--);
		}
	}

	delete [] tto_co; // clean up heap, compressed data is no longer needed

	// TTO savegame read and decompressed
	//////////////////////////////////////////////////////////

	m_progressline.SetPos(10);								// refresh progress bar
	m_doing = "Reformating, adjusting and completing data"; // refresh state information
	UpdateData(FALSE);

	//////////////////////////////////////////////////////////
	// 2. data reformation, adjustment and completion
	//
	// The overall savegame structures of TTO and TTD are the
	// same. Some parts can be copied without changing while
	// others need to be rewritten completely. Due to irregu-
	// larities in differences between TTO and TTD savegames,
	// the following data adjustment procedure has no simple
	// algorithm, but deals with each data segment separately.

	// adjust savegame title
	unsigned char ttd_title [47];	  // TTD savegame title
	memset(ttd_title, 0, 47);		  // fill it with zeros
	memcpy(ttd_title, tto_title, 39); // copy TTO savegame title to it

	// prepare for data adjustment
	unsigned char * ttd_dec = new unsigned char [618873]; // decompressed TTO data
	unsigned long tto_off = 0; // offset for reading TTO data
	unsigned long ttd_off = 0; // offset for writing TTD data
	const string nums [10] = {"1", "2", "3", "4", "5", "6", "7", "8", "9", "10"};
	unsigned long i, j; // array indices or loop counters
	// conversion variables, used in several data segments:
	unsigned char tto_color;
	const unsigned char ttd_color [16] = {0,14,3,4,11,7,12,1,8,6,9,13,15,5,10,2};
	unsigned short text_id;
	unsigned short town_ass;  // offset to the town something is associated with
	unsigned short veh_index; // index of a vehicle in the vehicle array
	unsigned char towns [80]; // town index shift
	unsigned char inds [100]; // industry index shift
	unsigned char industrytype;
	unsigned long pointer; // "pointer" to schedule string / town (global offset)

	int selected [10]; // removal selection in case of surplus towns/industries
	unsigned char id [100]; // town or industry id list (for surplus removal)

	// conversion catch: check if TTO savegame has more than 70 towns
	tto_off = 0x264;
	j = 0; // number of towns
	for (i = 0; i < 80; ++i)
	{
		if (tto_dec[tto_off] || tto_dec[tto_off + 1]) ++j; // town slot not empty
		tto_off += 0x4E;
	}
	if (j > 70) // too many towns
	{
		// find out which towns have stations assigned to them
		bool has_station [80]; // whether a town has stations or not
		for (i = 0; i < 80; ++i) has_station[i] = false;
		for (int k = 0; k < 200; ++k) // check all 200 stations
		{
			if (tto_dec[0x23BA6 + k*0x7E] || tto_dec[0x23BA6 + k*0x7E + 1]) // if active station slot
			{
				memcpy(&pointer, &tto_dec[0x23BA6 + k*0x7E + 2], 4); // pointer to its town
				has_station[(pointer - 0x264) / 0x4E] = true;
			}
		}

		// gather list of existing towns with no stations assigned to
		char ** list = new char * [j];
		unsigned long townparts;
		unsigned char listsize = 0; // number of removal-qualified towns
		tto_off = 0x264;
		for (i = 0; i < j; ++i)
		{
			if (!has_station[i]) // only list towns without stations assigned to
			{
				// determine town name data
				memcpy(&text_id,   &tto_dec[0x264 + i*0x4E + 4], 2);
				memcpy(&townparts, &tto_dec[0x264 + i*0x4E + 6], 4);

				// retrieve the town name string
				if (text_id == 0x10B6) // automatically generated name
				{
					list[listsize] = getTownName(tto_dec[0x496F9], townparts);
				}	
				else // user defined town name
				{
					char * name = new char [48]; // maximum length of an industry name is 47
					for (int k = 0; k < 48; ++k) name[k] = 0;

					memcpy(name, &tto_dec[0x45746 + 24*(text_id & 0x1FF)], 24); // copy custom string
					list[listsize] = name;
				}
				
				id[listsize] = i; // memorize town number
				++listsize;
			}
			tto_off += 0x4E;
		}
		int * order1 = bubbleSort(list, listsize); // sort the list

		if (listsize >= j-70) // enough towns to choose from
		{
			// launch too-many-towns window
			char * title = "Too many towns!";
			string text = "Since TTD allows only 70 towns, SVXConverter can't convert TTO savegames with more than that. In order to make this savegame convertible, ";
			text += nums[j - 71];
			text += " town";
			text += (j == 71 ? " needs" : "s need");
			text += " to be removed. This can only be done with towns that have no stations assigned to. Industries associated with ";
			text += (j == 71 ? "that town" : "those towns");
			text += " will be assigned to others if possible, and removed otherwise.\nSelect the one";
			text += (j == 71 ? "" : "s");
			text += " of your choice below (check ";
			text += (j == 71 ? "its" : "their");
			text += " identity within TTO if needed):";
			CSVXConverterSubWin TownDlg(title, (char *)(text.c_str()), list, listsize, j-70, selected, this);
			int button = TownDlg.DoModal();
			
			// clean up heap
			for (i = 0; i < listsize; ++i) delete [] list[i];
			delete [] list;
			
			if (button == IDCANCEL) // Cancel button pressed
			{
				m_progressline.SetPos(0); // reset progress bar
				m_doing = (ttofile + " could not be converted!").c_str();
				UpdateData(FALSE);
				delete [] tto_dec;
				delete [] ttd_dec;
				delete [] order1;
				return;
			}

			for (i = 0; i < j-70; ++i) removeTown(id[order1[selected[i]]], tto_dec); // remove the selected towns

			delete [] order1; // clean up heap
		}
		else // too few towns with no stations assigned to
		{
			MessageBox(("Since TTD allows only 70 towns, SVXConverter can't convert\nTTO savegames with more than that.\n\nIn order to make this savegame convertible, you need to\nmanually remove all stations associated with " + nums[j-71-listsize] + " town" + (j == 71+listsize ? "" : "s") + " within TTO.\nSVXConverter will then let you choose " + nums[j-71] + " town" + (j == 71 ? "" : "s") + " to be removed\nfrom the savegame, which can only be done with towns having\nno stations assigned to.").c_str(), "Savegame conversion failed!");

			m_progressline.SetPos(0); // reset progress bar
			m_doing = (ttofile + " could not be converted!").c_str();
			UpdateData(FALSE);
			delete [] tto_dec;
			delete [] ttd_dec;
			return;
		}
	}

	// conversion catch: check if TTO savegame has more than 90 industries
	tto_off = 0x29E16;
	j = 0; // number of industries
	for (i = 0; i < 100; ++i)
	{
		if (tto_dec[tto_off] || tto_dec[tto_off + 1]) ++j; // industry slot not empty
		tto_off += 0x28;
	}
	if (j > 90) // too many industries
	{
		const string ind_eng [12] = {" Coal Mine"," Power Station"," Sawmill"," Forest"," Oil Refinery"," Oil Rig",
									 " Factory"," Steel Mill"," Farm"," Iron Ore Mine"," Oil Wells"," Bank"};
		const string ind_ger [12] = {" Kohlebergwerk"," Kraftwerk"," Sgewerk"," Wald"," lraffinerie"," lbohrinsel",
									 " Fabrik"," Stahlwalzwerk"," Bauernhof"," Eisenerzbergwerk"," lpumpanlagen"," Bank"};

		// gather list of existing industries
		char ** list = new char * [j];
		unsigned long townparts;
		bool has_dock;				// whether an oil rig has a dock or not
		unsigned short xy;			// position of the industry
		unsigned char listsize = 0; // number of removal-qualified industries
		tto_off = 0x29E16;
		for (i = 0; i < j; ++i)
		{
			industrytype = tto_dec[tto_off + 0x24]; // read TTO industry type
			
			if (industrytype == 5) // if oil rig: check if it has a dock
			{
				has_dock = false;
				memcpy(&xy, &tto_dec[tto_off], 2);
				if ((tto_dec[0x4971D + xy] & 0xF0) == 0x50) has_dock = true;
			}
			if (industrytype != 5 || !has_dock) // not an oil rig with a dock
			{
				// determine town name data
				memcpy(&town_ass, &tto_dec[0x29E16 + i*0x28 + 2], 2); // town the industry is assigned to
				memcpy(&text_id,   &tto_dec[town_ass + 4], 2);
				memcpy(&townparts, &tto_dec[town_ass + 6], 4);

				// retrieve the town name string
				if (text_id == 0x10B6) // automatically generated name
				{
					list[listsize] = getTownName(tto_dec[0x496F9], townparts);
				}	
				else // user defined town name
				{
					char * name = new char [48]; // maximum length of an industry name is 47
					for (int k = 0; k < 48; ++k) name[k] = 0;

					memcpy(name, &tto_dec[0x45746 + 24*(text_id & 0x1FF)], 24); // copy custom string
					list[listsize] = name;
				}

				// append industry type name (in german if german town names)
				if (tto_dec[0x496F9] == 1) // german
					memcpy(list[listsize] + string(list[listsize]).length(), ind_ger[industrytype].c_str(), ind_ger[industrytype].length());
				else // english or american
					memcpy(list[listsize] + string(list[listsize]).length(), ind_eng[industrytype].c_str(), ind_eng[industrytype].length());

				id[listsize] = i; // memorize industry number
				++listsize;
			}

			tto_off += 0x28;
		}
		int * order2 = bubbleSort(list, listsize); // sort the list

		// launch too-many-industries window
		char * title = "Too many industries!";
		string text = "Since TTD allows only 90 industries, SVXConverter can't convert TTO savegames with more than that. In order to make this savegame convertible, ";
		text += nums[j-91];
		text += " industr";
		text += (j == 91 ? "y needs" : "ies need");
		text += " to be removed. Note that this can't be done with oil rigs that have a dock.\n\nSelect the one";
		text += (j == 91 ? "" : "s");
		text += " of your choice below (check ";
		text += (j == 91 ? "its" : "their");
		text += " identity within TTO if needed):";
		CSVXConverterSubWin IndDlg(title, (char *)(text.c_str()), list, listsize, j-90, selected, this);
		int button = IndDlg.DoModal();

		// clean up heap
		for (i = 0; i < listsize; ++i) delete [] list[i];
		delete [] list;
		
		if (button == IDCANCEL) // Cancel button pressed
		{
			m_progressline.SetPos(0); // reset progress bar
			m_doing = (ttofile + " could not be converted!").c_str();
			UpdateData(FALSE);
			delete [] tto_dec;
			delete [] ttd_dec;
			delete [] order2;
			return;
		}

		for (i = 0; i < j-90; ++i) removeIndustry(id[order2[selected[i]]], tto_dec); // remove the selected industries

		delete [] order2; // clean up heap
	}

	// copy date & daytime (4 bytes)
	memcpy(ttd_dec, tto_dec, 4);

	// copy text effects (30 entries, 20 bytes each)
	tto_off = ttd_off = 4;
	for (i = 0; i < 30; ++i)
	{
		// adjust text ID
		memcpy(&text_id, &tto_dec[tto_off], 2);
		if (text_id != 0xFFFF) text_id += 0x400; // cost or income
		memcpy(&ttd_dec[ttd_off], &text_id, 2);
		
		memcpy(&ttd_dec[ttd_off + 2], &tto_dec[tto_off + 2], 18);
		tto_off += 20;
		ttd_off += 20;
	}

	// copy random generator seed (8 bytes)
	tto_off = ttd_off = 0x25C;
	memcpy(&ttd_dec[ttd_off], &tto_dec[tto_off], 8);

	// town array (TTD: 70 entries, 0x5E bytes each)
	tto_off = ttd_off = 0x264;
	j = 0; // number of towns
	for (i = 0; i < 80; ++i)
	{
		if (tto_dec[tto_off] || tto_dec[tto_off + 1]) // town slot not empty
		{
			towns[i] = j; // memorize town index shift
			++j;
			memcpy(&ttd_dec[ttd_off], &tto_dec[tto_off], 0x4C); // copy 76 bytes
			
			// adjust town name
			memcpy(&text_id, &ttd_dec[ttd_off + 4], 2);
			if (text_id == 0x10B6) text_id = 0x20C1; // auto-generated town name
			else text_id += 11052; // custom town name
			memcpy(&ttd_dec[ttd_off + 4], &text_id, 2);

			memset(&ttd_dec[ttd_off + 0x4C], 0, 8); // food & water delivery = 0
			memcpy(&ttd_dec[ttd_off + 0x54], &tto_dec[tto_off + 0x4C], 2);
			memset(&ttd_dec[ttd_off + 0x56], 0, 8); // 8 zeros: meaning unknown (fillers?)
			ttd_off += 0x5E;
		}
		else
		{
			towns[i] = 0; // no town in this slot
		}
		tto_off += 0x4E;
	}
	for (i = j; i < 70; ++i) // fill the remaining slots with 0
	{
		memset(&ttd_dec[ttd_off], 0, 0x5E);
		ttd_off += 0x5E;
	}

	// copy schedule strings heap
	tto_off = 0x1AC4;
	ttd_off = 0x1C18;
	memcpy(&ttd_dec[ttd_off], &tto_dec[tto_off], 6000); // copy all 3000 entries
	memset(&ttd_dec[ttd_off + 6000], 0, 4000); // TTD stores 2000 more (set to 0)
	
	// copy animated tiles (0x200 bytes)
	tto_off = 0x3234;
	ttd_off = 0x4328;
	memcpy(&ttd_dec[ttd_off], &tto_dec[tto_off], 0x200);

	// copy pointer to the end of schedules
	tto_off = 0x3434;
	ttd_off = 0x4528;
	memcpy(&pointer, &tto_dec[tto_off], 4);
	pointer += 0x154;
	memcpy(&ttd_dec[ttd_off], &pointer, 4);

	// depot array (TTD: 255 entries, 6 bytes each)
	tto_off = 0x3438;
	ttd_off = 0x452C;
	for (i = 0; i < 252; ++i) // TTO has only 252 depots
	{
		memcpy(&ttd_dec[ttd_off], &tto_dec[tto_off], 2);

		// convert town association
		memcpy(&town_ass, &tto_dec[tto_off + 2], 2);
		if (town_ass) town_ass = towns[(town_ass - 0x264) / 0x4E] * 0x5E + 0x264;
		memcpy(&ttd_dec[ttd_off] + 2, &town_ass, 2);
		memset(&ttd_dec[ttd_off] + 4, 0, 2);

		tto_off += 6;
		ttd_off += 6;
	}
	memset(&ttd_dec[ttd_off], 0, 18); // TTD stores 3 more depots (6 bytes = 0 each)

	// copy processing counters (0xE bytes)
	tto_off = 0x3A20;
	ttd_off = 0x4B26;
	memcpy(&pointer, &tto_dec[tto_off], 4);
	pointer = towns[(pointer - 0x264) / 0x4E] * 0x5E + 0x264;
	if (pointer >= 0x264 + j * 0x5E) pointer = 0x264;
	memcpy(&ttd_dec[ttd_off], &pointer, 4);

	memcpy(&ttd_dec[ttd_off + 4], &tto_dec[tto_off + 4], 0xA);

	// copy various costs (TTD: 49, TTO: 48)
	tto_off = 0x3A2E;
	ttd_off = 0x4B34;

	union
	{
		__int64 full64;
		struct
		{
			short part16;
			long  part32;
			short filler;
		};
	} val;

	for (i = 0; i < 49; ++i)
	{
		if (i == 25) // field tile clearing cost inexistent in TTO
		{
			memcpy(&ttd_dec[ttd_off], &ttd_dec[0x4B58], 6);
		}
		else
		{
			val.full64 = 0;
			// read cost value from TTO:
			memcpy(&(val.part32), &tto_dec[tto_off], 4);
			memcpy(&(val.part16), &tto_dec[tto_off + 4], 2);
			// make more expensive if necessary (mean approximation!):
			if (i == 15) val.full64 *= 20.0/3.0; // new rail engine
			else if (i == 17) val.full64 *= 10.0; // new aircraft
			// write cost value to TTD:
			memcpy(&ttd_dec[ttd_off], &(val.part32), 4);
			memcpy(&ttd_dec[ttd_off + 4], &(val.part16), 2);
			tto_off += 6;
		}
		ttd_off += 6;
	}

	// copy cargo delivery payment factors
	tto_off = 0x3B4E;
	ttd_off = 0x4C5A;
	for (i = 0; i < 11; ++i)
	{
		val.full64 = 0;
		memcpy(&(val.part32), &tto_dec[tto_off], 4);
		memcpy(&(val.part16), &tto_dec[tto_off + 4], 2);

		// Increase them to compensate for the faster time advance in TTD compared to TTO
		// which otherwise would cause much less income while the annual running costs of
		// the vehicles stay the same. The correct multiplier would be 1.5 because a day
		// in TTO takes three seconds while a day in TTD only takes 2 seconds. The
		// empirical bonus of 0.28 compensates the property maintenance costs introduced
		// in TTD.
		val.full64 *= 1.78;

		memcpy(&ttd_dec[ttd_off], &(val.part32), 4);
		memcpy(&ttd_dec[ttd_off + 4], &(val.part16), 2);
		memset(&ttd_dec[ttd_off + 6], 0, 2); // filler
		tto_off += 8;
		ttd_off += 8;
	}
	memcpy(&ttd_dec[ttd_off], &ttd_dec[0x4CA2], 8); // TTD has 1 more cargo type

	// convert landscape arrays 1, 2, 4 & 5 (TTO) to 1 through 5 (TTD)
	tto_off = 0x3BA6;
	ttd_off = 0x4CBA;
	unsigned char l1_tto, l2_tto, l4_tto, l5_tto; // read elements of arrays L1,2,4,5
	unsigned char l1_ttd, l2_ttd, l4_ttd, l5_ttd; // elements for TTD arrays L1,2,4,5
	unsigned short l3_ttd; // element for TTD L3 (inexistent in TTO)
	unsigned char tileclass; // tile class (0 to 11)
	bool bridge_dir = false; // bridge direction (x: false, y: true)
	// this for loop determines the industry index shifts (used for class 8 tile conversion)
	j = 0;
	for (i = 0; i < 100; ++i)
	{
		if (tto_dec[0x29E16 + i*0x28] || tto_dec[0x29E17 + i*0x28]) // industry slot not empty
		{
			inds[i] = j; // memorize industry index shift
			++j;
		}
		else
		{
			inds[i] = 0; // no industry in this slot
		}
	}
	for (i = 0; i < 0x10000; ++i)
	{
		// read elements out of TTO arrays
		l1_tto = tto_dec[0x03BA6 + i];
		l2_tto = tto_dec[0x13BA6 + i];
		l4_tto = tto_dec[0x4971D + i];
		l5_tto = tto_dec[0x5971D + i];

		// convert them to TTD array elements
		tileclass = l4_tto / 16; // tile class stored in upper 4 bits in L4
		l1_ttd = l1_tto;
		l2_ttd = l3_ttd = l5_ttd = 0; // elements of L2, L3 & L5 = 0 by default
		l4_ttd = l4_tto;
		if (tileclass == 0) // free land
		{
			// no information in L2
			// L3 = 0, because no fences in TTO
			l5_ttd = l5_tto; // L5 stays the same
		}
		else if (tileclass == 1 || tileclass == 11) // railway (1) or monorail (11)
		{
			if (tileclass == 11) // monorail
			{
				l3_ttd = 1; // set track type in L3: monorail
				l4_ttd -= 0xA0; // convert tile class 11 to 1
			}
			if (l5_tto / 64 == 0) // L5 bits 7-6 = 0: track without signal
			{
				l2_ttd = l2_tto; // L2 stays the same
				if (tileclass == 11) ++l2_ttd; // monorail: bare land -> grass
				l5_ttd = l5_tto; // L5 stays the same
			}
			else if (l5_tto / 64 == 1) // L5 bits 7-6 = 1: track with signal
			{
				l2_ttd = l2_tto + (l5_tto & 24) * 8; // TTO L5 B4-3 -> TTD L2 B7-6
				if (tileclass == 11) ++l2_ttd; // monorail: bare land -> grass
				l3_ttd += 192; // bits 7-6 set: signals on both sides of the track
				l5_ttd = 64; // bit 6 set: track with signal
				l5_ttd += ((l5_tto & 32) == 32) + 1; // TTO bit 5: track direction
			}
			else // depot
			{
				// no information in L2
				l5_ttd = l5_tto; // L5 stays the same
			}
		}
		else if (tileclass == 2) // road (depot) or level crossing
		{
			if (l5_tto / 16 == 0) // bits 7-4 clear: road
			{
				l2_ttd = l2_tto;
				if (l2_tto == 4) ++l2_ttd; // with small trees: inexistent in TTD
				// L3 = 0, because no snow or desert
			}
			else if (l5_tto / 16 == 1) // bits 7-5 clear, 4 set: level crossing
			{
				l2_ttd = l2_tto; // L2 stays the same
				l3_ttd = ttd_dec[0x4CBA + i]; // owner
				// bits 15-8 of L3 = 0, (only railway possible & no snow or desert)
			}
			else // road depot
			{
				// no information in L2
				// L3 = 0, because no snow or desert
			}
			l5_ttd = l5_tto; // L5 stays the same for all types
		}
		else if (tileclass == 3) // town building
		{
			l2_ttd = (l2_tto & 63); // copy bits 5-0 (building type)
			if (l2_ttd >= 5) ++l2_ttd;
			l3_ttd = (l2_tto & 192); // stage of construction: bits 7-6: L2 -> L3
			l5_ttd = l5_tto; // L5 stays the same
		}
		else if (tileclass == 4) // tree(s)
		{
			l2_ttd = l2_tto; // L2 stays the same
			l3_ttd = (l5_tto & 56) / 8; // type of tree in TTO: L5 bits 5-3
			l5_ttd = (l5_tto & 199); // copy L5 bits 7-6, 2-0
		}
		else if (tileclass == 5) // station
		{
			l2_ttd = l2_tto; // L2 stays the same
			l3_ttd = (l5_tto >= 8 && l5_tto <= 0xF); // 1: monorail, 0: other
			l5_ttd = l5_tto;
			if (l5_tto >= 8) l5_ttd -= 8; // not conventional railway
			if (l5_tto >= 0x4A) ++l5_ttd; // skip heliport
		}
		else if (tileclass == 6) // water related
		{
			if (i < 256 || i >= 254*256 || i % 256 == 0 || i % 256 >= 254)
				l2_ttd = 0x10; // map border
			// no information in L3
			l5_ttd = l5_tto; // L5 stays the same
		}
		else if (tileclass == 7) // void (S/E & S/W map border)
		{
			l2_ttd = 0x10; // L2 stays the same
			// no information in L3
			l5_ttd = 3;
		}
		else if (tileclass == 8) // industry
		{
			l2_ttd = inds[l2_tto]; // shift industry index if necessary
			// no information in L3
			l5_ttd = l5_tto;
			if		(l5_tto == 0x24) ++l5_ttd; // farm: silo -> silo
			else if (l5_tto == 0x25) --l5_ttd; // farm: cattle shed -> barn
			else if (l5_tto == 0x27) --l5_ttd; // farm: harvested field -> cattle shed
			else if (l5_tto >= 0x2C) l5_ttd += 3; // iron ore mine, steel mill or bank
			else if (l5_tto >= 0x28) --l5_ttd; // factory
		}
		else if (tileclass == 9) // tunnel entrance/exit or bridge
		{
			if (l5_tto / 128) // bit 7 set: bridge
			{
				l2_ttd = l2_tto;
				if ((l5_tto & 6) == 6) l2_ttd += 0xA0; // L5 bits 2-1 = 3: monorail
				if (l5_tto & 64) // bit 6 also set: middle part
				{
					l3_ttd = ((l5_tto & 4) == 4) * 16; // L3 bits 7-4: track type
					l3_ttd += ((l5_tto & 24) == 24); // track type under bridge
					l5_ttd = (l5_tto & 232); // L5: copy bits 7-5,3
					if (l5_tto & 16) // if bit 4 set: monorail under bridge
						l5_ttd &= 231; // erase L5 bits 4-3
					if (l5_tto & 4) // if L5 bit 2 set: monorail bridge
					{
						bridge_dir = l5_tto & 1; // L5 bit 0: bridge direction
						if ((tto_dec[0x5971D + i + 1 + 255 * bridge_dir] & 64) == 0)
							// next to south ending
							l2_ttd = (l2_ttd & 240) + 1;
					}
				}
				else // bridge ending
				{
					l3_ttd = ((l5_tto & 6) == 6); // L3 bits 3-0: track type
					l5_ttd = (l5_tto & 224); // L5: copy bits 7-5
				}
				l5_ttd += (l5_tto & 1); // copy bit 0
				l5_ttd += ((l5_tto & 6) == 2) * 2; // bits 2-1: track type
			}
			else // bit 7 clear: tunnel entrance/exit
			{
				// no information in L2
				l3_ttd = ((l5_tto & 8) == 8); // if TTO L5 bit 3 set: monorail
				l5_ttd = (l5_tto & 3); // L5: copy bits 1-0 (direction)
				l5_ttd += ((l5_tto & 12) == 4) * 4; // if L5 bits 3-2 = 1: road
			}
		}
		else if (tileclass == 10) // special tile
		{
			// no information in L2 & L3
			l5_ttd = l5_tto; // L5 stays the same
		}

		// shift town index (tile owner in L1) if necessary
		if (tileclass != 3 && tileclass != 7 && tileclass != 8 && l1_tto >= 128)
			l1_ttd = towns[l1_tto - 128] + 128;

		// write elements into TTD arrays
		ttd_dec[0x04CBA + i] = l1_ttd;
		ttd_dec[0x14CBA + i] = l2_ttd;
		memcpy(&ttd_dec[0x24CBA + 2*i], &l3_ttd, 2);
		ttd_dec[0x77179 + i] = l4_ttd;
		ttd_dec[0x87179 + i] = l5_ttd;
	}

	m_progressline.SetPos(60); // refresh progress bar
	UpdateData(FALSE);

	// create rainforest/desert area array
	// (since there is no rainforest/desert in TTO, just set all values to 0)
	tto_off = 0x23BA6;
	ttd_off = 0x44CBA;
	memset(&ttd_dec[ttd_off], 0, 0x4000);

	// station array (TTD: 250 entries, 0x8E bytes each)
	tto_off = 0x23BA6; // still the same
	ttd_off = 0x48CBA;
	unsigned short facilities [8] = { // number of station facilities, for company value calc.
		0, 0, 0, 0, 0, 0, 0, 0};
	unsigned char components; // number of facilities of the current station
	const unsigned char cargo12 [8] = {0,0,0,175,255,0,0,255}; // 12th cargo entry
	unsigned short airporttype;
	for (j = 0; j < 200; ++j) // copy all 200 entries from TTO
	{
		if (tto_dec[tto_off + 0x77] < 8) // active station entry? (only for company value)
		{
			components = tto_dec[tto_off + 0x78];
			for (i = 0; i < 5; ++i) // 5 possible facilities
			{ 
				facilities[tto_dec[tto_off + 0x77]] += (components&1); // facility present?
				components >>= 1; // shift to next facility bit
			}
		}

		memcpy(&ttd_dec[ttd_off], &tto_dec[tto_off], 0x74);

		if (tto_dec[tto_off] || tto_dec[tto_off + 1])
		{
			// convert town association
			memcpy(&town_ass, &tto_dec[tto_off + 2], 2);
			town_ass = towns[(town_ass - 0x264) / 0x4E] * 0x5E + 0x264;
			memcpy(&ttd_dec[ttd_off + 2], &town_ass, 2);

			// adjust station name
			memcpy(&text_id, &ttd_dec[ttd_off + 0x14], 2);
			if (text_id / 256 == 83) text_id += 0x2C2C; // custom station name
			else text_id += 0x1800; // auto-generated station name
			memcpy(&ttd_dec[ttd_off + 0x14], &text_id, 2);
		}

		 // write 12th cargo entry
		if (tto_dec[tto_off] || tto_dec[tto_off + 1])
			memcpy(&ttd_dec[ttd_off + 0x74], cargo12, 8);
		else
			memset(&ttd_dec[ttd_off + 0x74], 0, 8);

		memcpy(&ttd_dec[ttd_off + 0x7C], &tto_dec[tto_off + 0x74], 5);
		
		// determine whether airport = small, large or oil rig
		memcpy(&airporttype, &tto_dec[tto_off + 0x79], 2);
		if (airporttype & 256) // 8th bit set -> large airport
			ttd_dec[ttd_off + 0x81] = 1; // write 1
		else if (airporttype & 64) // 6th bit set -> oil rig
			ttd_dec[ttd_off + 0x81] = 3; // write 3
		else // small airport
			ttd_dec[ttd_off + 0x81] = 0; // write 0

		memcpy(&ttd_dec[ttd_off + 0x82], &tto_dec[tto_off + 0x7B], 3);
		memset(&ttd_dec[ttd_off + 0x85], 0, 1); // write 0, meaning unknown
		
		// copy airport status (to be adjusted in vehicle array processing)
		memcpy(&ttd_dec[ttd_off + 0x86], &tto_dec[tto_off + 0x79], 2);
		
		// write 6 zeros of which the last 4 have no known meaning (fillers?)
		memset(&ttd_dec[ttd_off + 0x88], 0, 6);

		tto_off += 0x7E;
		ttd_off += 0x8E;
	}
	memset(&ttd_dec[ttd_off], 0, 50*0x8E); // write additional 50 empty entries

	// industry array (TTD: 90 entries, 0x36 bytes each)
	tto_off = 0x29E16;
	ttd_off = 0x51766;
	j = 0; // number of industries
	unsigned char current_year = tto_dec[0x496EA]; // read current year
	for (i = 0; i < 100; ++i)
	{
		if (tto_dec[tto_off] || tto_dec[tto_off + 1]) // industry slot not empty
		{
			memcpy(&ttd_dec[ttd_off], &tto_dec[tto_off], 0xA);

			// convert town association
			memcpy(&town_ass, &tto_dec[tto_off + 2], 2);
			town_ass = towns[(town_ass - 0x264) / 0x4E] * 0x5E + 0x264;
			memcpy(&ttd_dec[ttd_off + 2], &town_ass, 2);

			// convert 2 8bit ints to 16bit (waiting cargo amounts):
			ttd_dec[ttd_off + 0xA] = tto_dec[tto_off + 0xA];
			ttd_dec[ttd_off + 0xB] = 0;
			ttd_dec[ttd_off + 0xC] = tto_dec[tto_off + 0xB];
			ttd_dec[ttd_off + 0xD] = 0;

			memcpy(&ttd_dec[ttd_off + 0xE], &tto_dec[tto_off + 0xC], 0x18);

			// adjust industry type
			industrytype = tto_dec[tto_off + 0x24];
			if (industrytype > 6) ++industrytype;
			if (industrytype == 0xA) industrytype = 0x12;
			ttd_dec[ttd_off + 0x26] = industrytype;

			ttd_dec[ttd_off + 0x27] = tto_dec[tto_off + 0x26]; // owner

			// adjust color palette
			tto_color = tto_dec[tto_off + 0x27];
			ttd_dec[ttd_off + 0x28] = ttd_color[tto_color];

			// write last year anything was produced
			ttd_dec[ttd_off + 0x29] = current_year;

			// write production counter (TTD: 16bit, TTO: 8bit)
			ttd_dec[ttd_off + 0x2A] = tto_dec[tto_off + 0x25];
			ttd_dec[ttd_off + 0x2B] = 0;

			// write 10 zeros of which the last 9 have no known meaning (fillers?)
			memset(&ttd_dec[ttd_off + 0x2C], 0, 10);

			++j;
			ttd_off += 0x36;
		}
		tto_off += 0x28;
	}
	memset(&ttd_dec[ttd_off], 0, (90-j)*0x36); // fill the remaining slots with 0

	// company array (TTD: 8 entries, 0x3B2 bytes each)
	tto_off = 0x2ADB6;
	ttd_off = 0x52A62;
	signed long company_value;		// used for company value calculation
	signed long vehicle_values [8] = {0, 0, 0, 0, 0, 0, 0, 0}; // sum of all vehicle values
	signed long vehicle_value;		// value of the current vehicle
	unsigned short date;			// current date
	memcpy(&date, tto_dec, 2);		// read current date

	for (i = 0; i < 850; ++i) // go through the whole vehicle array
	{
		if (tto_dec[0x2C746 + 0x80*i + 0x23] < 8) // active vehicle entry?
		{
			memcpy(&vehicle_value, &tto_dec[0x2C746 + 0x80*i + 0x58], 4); // read vehicle value
			vehicle_values[tto_dec[0x2C746 + 0x80*i + 0x23]] += vehicle_value; // sum up values
		}
	}

	for (j = 0; j < 8; ++j) // 8 companies
	{
		memcpy(&company_value, &tto_dec[tto_off + 0x10], 4); // read company cash for later use
		memcpy(&ttd_dec[ttd_off], &tto_dec[tto_off], 0x22); // copy various company information

		// adjust manager's face if necessary
		memcpy(&i, &tto_dec[tto_off + 6], 4); // read face data (32 bits)
		if (((i & 0x8000000) && (((i >> 26) & 1) == ((i >> 19) & 1))))
		{
			// if face would be black in TTD, adjust tie color and thereby face color
			i &= 0xF7FFFFFF; // set bit 27 to 0
			memcpy(&ttd_dec[ttd_off + 6], &i, 4); // write adjusted face data
		}

		// adjust company name (text ID)
		memcpy(&text_id, &ttd_dec[ttd_off], 2);
		if (text_id == 0x4C00) text_id = 0x7000;		   // company name: "Unnamed"
		else if (text_id / 256 == 0x52) text_id += 0x2B2C; // custom name
		else if (text_id) text_id += 0x240D;			   // auto-generated name
		memcpy(&ttd_dec[ttd_off], &text_id, 2);

		// adjust manager's name (text ID)
		memcpy(&text_id, &ttd_dec[ttd_off + 0xA], 2);
		if (text_id > 6) text_id += 0x2B2C;			   // custom name
		memcpy(&ttd_dec[ttd_off + 0xA], &text_id, 2);
		
		// adjust company color
		tto_color = tto_dec[tto_off + 0x18]; // read TTO color
		ttd_dec[ttd_off + 0x18] = ttd_color[tto_color]; // convert it to TTD & write it

		memcpy(&ttd_dec[ttd_off + 0x22], &tto_dec[tto_off + 0x22], 2); // cargo types delivered
		memset(&ttd_dec[ttd_off + 0x24], 0, 2); // write two zeros (16bit -> 32bit)
		tto_off += 0x24;
		ttd_off += 0x26;

		for (i = 0; i < 3; ++i) // copy all tables of expenses (3 years)
		{
			memcpy(&ttd_dec[ttd_off], &tto_dec[tto_off], 24); // copy 6 values, 32bit each
			memset(&ttd_dec[ttd_off + 24], 0, 4); // write additonal value: property maintenance
			memcpy(&ttd_dec[ttd_off + 28], &tto_dec[tto_off + 24], 24); // copy 6 values
			tto_off += 12*4;
			ttd_off += 13*4;
		}

		// calculate company value (not stored in TTO)
		// company value = cash + 1.5 * vehicle values + 2500 * station facilities
		company_value += (3 * vehicle_values[j]) / 2;
		company_value += 2500l * (long)facilities[j];

		for (i = 0; i < 25; ++i) // copy performance array (25 quarters)
		{
			memcpy(&ttd_dec[ttd_off], &tto_dec[tto_off], 16); // copy 4 values, 32bit each
			memcpy(&ttd_dec[ttd_off + 16], &company_value, 4); // TTD-only value: company value
			tto_off += 4*4;
			ttd_off += 5*4;
		}

		// copy current AI activity data & various other stuff
		memcpy(&ttd_dec[ttd_off], &tto_dec[tto_off], 0xED);

		// adjust AI activity data
		if (ttd_dec[ttd_off + 5] >= 6 && ttd_dec[ttd_off + 5] <= 9) // building conventional railway
			ttd_dec[ttd_off + 0xE9] = 0; // railtype to use = conventional railway
		else if (ttd_dec[ttd_off + 5] >= 10 && ttd_dec[ttd_off + 5] <= 13) // building monorail
			ttd_dec[ttd_off + 0xE9] = 1; // railtype to use = monorail
		if (ttd_dec[ttd_off + 5] >= 10) // building monorail, road, ships, aircraft etc.
			ttd_dec[ttd_off + 5] -= 4; // TTD has no explicit monorail states
		
		// determine, which track types are accessible
		if ((tto_dec[0x496D0] >> j) & 1) // monorail accessible to this company?
		{
			if (date >= 36965) // current date >= introduction date of 1st maglev?
				ttd_dec[ttd_off + 0xED] = 3; // all track types accessible
			else
				ttd_dec[ttd_off + 0xED] = 2; // conventional railway & monorail accessible
		}
		else
		{
			ttd_dec[ttd_off + 0xED] = 1; // only railway accessible
		}

		// fix invalid railtypes to use (value seems to be unused in TTO & is invalid for TTD)
		if (ttd_dec[ttd_off + 0xE9] > 0)
			ttd_dec[ttd_off + 0xE9] = (ttd_dec[ttd_off + 0xED] == 1 ? 0 : 1);

		memcpy(&ttd_dec[ttd_off + 0xEE], &tto_dec[tto_off + 0xEC], 2); // XY of company HQ
		memset(&ttd_dec[ttd_off + 0xF0], 0xFF, 4); // companies owning shares (0xFF = none)
		memset(&ttd_dec[ttd_off + 0xF4], 0, 8);	// write 8 zeros, meaning unknown (fillers?)
		tto_off += 0xEE;
		ttd_off += 0xFC;
	}

	// vehicle array (TTD: 850 entries, 0x80 bytes each)
	tto_off = 0x2C746;
	ttd_off = 0x547F2;
	unsigned short sprite;	  // used for sprite number calculation
	unsigned short vehid_tto; // used for vehicle type ID calculation
	const unsigned short vehid_ttd [103] = {0,0,8,8,8,8,8,9,10,11,12,13,14,15,15,22,23,24,25,
		26,27,29,28,30,31,32,33,34,35,36,37,55,57,59,58,60,61,62,63,64,65,66,67,116,116,117,
		118,123,124,126,127,132,133,135,136,138,139,141,142,144,145,147,148,150,151,153,154,
		204,205,206,207,208,211,212,211,212,211,212,215,216,217,218,219,220,221,222,223,224,
		225,226,227,228,229,230,231,232,233,234,235,236,237,238,253}; // see separate doc.
	unsigned char spriteset; // used for sprite set offset calculation
	const unsigned char spriteset_rail [41] = {0,2,4,4,8,10,12,14,16,18,20,22,40,42,44,46,48,
		52,54,66,68,70,72,74,76,78,80,82,84,86,120,122,124,126,128,130,132,134,136,138,140};
	unsigned char air_pos, air_state;	// aircraft movement states
	unsigned short air_x, air_y;		// aircraft X/Y position (tile index * 16 + offset)
	unsigned char * target_airport = 0;	// points to the target airport in the station array
	j = 0;
	for (i = 0; i < 800; ++i) // copy all 800 vehicles from TTO
	{
		if (i == 640) // when all 640 company-owned vehicles have been copied ...
		{
			// ... write additional 50 company vehicles ...
			for (j = 0; j < 50; ++j)
			{
				memset(&ttd_dec[ttd_off], 0, 4);
				ttd_dec[ttd_off + 4] = j + 0x80;
				ttd_dec[ttd_off + 5] = 2;
				memset(&ttd_dec[ttd_off + 6], 0, 0x7A);
				ttd_off += 0x80;
			}
			// ... and proceed with last 160 entries (special vehicles)
		}

		// vehicle class (1 byte)
		ttd_dec[ttd_off] = tto_dec[tto_off]; // copy value
		if (tto_dec[tto_off] >= 32 && tto_dec[tto_off] <= 36)
			ttd_dec[ttd_off] -= 16; // adjust: various
		else if (tto_dec[tto_off] == 37)
			ttd_dec[ttd_off] = 16; // adjust: monorail
		else if (tto_dec[tto_off] == 38)
			ttd_dec[ttd_off] = 21; // adjust: disaster vehicle

		// various (0x16 bytes)
		memcpy(&ttd_dec[ttd_off + 1], &tto_dec[tto_off + 1], 0x16);

		// adjust bounding block share index
		if (ttd_dec[ttd_off])
		{
			memcpy(&veh_index, &tto_dec[tto_off + 2], 2);		
			if (veh_index >= 0x0280 && veh_index != 0xFFFF)
			{
				veh_index += 50;
				memcpy(&ttd_dec[ttd_off + 2], &veh_index, 2);
			}
		}
		
		if (j) // currently processed entry one of the last 160?
		{
			// adjust vehicle index
			memcpy(&veh_index, &tto_dec[tto_off + 4], 2);
			veh_index += 50;
			memcpy(&ttd_dec[ttd_off + 4], &veh_index, 2);
		}

		// adjust pointer to schedules
		if (ttd_dec[ttd_off] > 0 && ttd_dec[ttd_off] < 20)
		{
			memcpy(&pointer, &tto_dec[tto_off + 0x6], 4);
			pointer += 0x154;
			memcpy(&ttd_dec[ttd_off + 0x6], &pointer, 4);
		}

		// cycle counter (1 byte)
		ttd_dec[ttd_off + 0x17] = tto_dec[tto_off + 0x3B]; // different position in TTO

		// maximum speed (TTD: 16bit, TTO: 8bit)
		ttd_dec[ttd_off + 0x18] = tto_dec[tto_off + 0x17];
		ttd_dec[ttd_off + 0x19] = 0;

		// various (0xE bytes)
		memcpy(&ttd_dec[ttd_off + 0x1A], &tto_dec[tto_off + 0x18], 0xE);

		// curent sprite (16 bits)
		memcpy(&sprite, &tto_dec[tto_off + 0x26], 2); // read current sprite
		if (sprite >= 1460 && sprite <= 2538)
		{
			if		(sprite <= 1464) // aircraft smoke puff
				sprite += 580;
			else if	(sprite <= 1899) // coaches part 1
				sprite += 857;
			else if (sprite <= 1939) // coaches part 2
				sprite += 861;
			else if (sprite <= 1963) // monorail coaches part 1
				sprite += 889;
			else if (sprite <= 2003) // monorail coaches part 2
				sprite += 893;
			else if (sprite <= 2027) // steam engines
				sprite += 901;
			else if (sprite <= 2035) // steam engine, doesn't exist in TTD
				sprite += 893;
			else if (sprite <= 2087) // dieses & electric engines
				sprite += 893;
			else if (sprite <= 2095) // monorail engine
				sprite += 897;
			else if (sprite <= 2114) // special effects part 1
				sprite += 977;
			else if (sprite <= 2162) // (semi-)modern buses & trucks part 1
				sprite += 977;
			else if (sprite <= 2242) // (semi-)modern buses & trucks part 2
				sprite += 985;
			else if (sprite <= 2290) // old buses & trucks part 1
				sprite += 1041;
			else if (sprite <= 2370) // old buses & trucks part 2
				sprite += 1049;
			else if (sprite <= 2379) // oil tanker OR goods ship
				sprite += 1297;		 // to be readjusted later if goods ship
			else if (sprite <= 2395) // ships
				sprite += 1305;
			else if (sprite <= 2435) // special effects part 2
				sprite += 1305;
			else if (sprite <= 2515) // aircraft
				sprite += 1305;
			else					 // rotor or disaster-related vehicles
				sprite += 1385;
			if (sprite >= 3669 && sprite <= 3676) // if oil tanker or goods ship
			{
				if (tto_dec[tto_off + 0x36] != 3) // if cargo type != oil
					sprite += 8; // make it a goods ship
			}
		}
		memcpy(&ttd_dec[ttd_off + 0x28], &sprite, 2); // write adjusted sprite number

		// various (0xA bytes)
		memcpy(&ttd_dec[ttd_off + 0x2A], &tto_dec[tto_off + 0x28], 0xA);

		// curent speed (TTD: 16bit, TTO: 8bit)
		ttd_dec[ttd_off + 0x34] = tto_dec[tto_off + 0x32];
		ttd_dec[ttd_off + 0x35] = 0;

		// various (4 bytes)
		memcpy(&ttd_dec[ttd_off + 0x36], &tto_dec[tto_off + 0x33], 4);

		// cargo capacity (TTD: 16bit, TTO: 8bit)
		ttd_dec[ttd_off + 0x3A] = tto_dec[tto_off + 0x37];
		ttd_dec[ttd_off + 0x3B] = 0;

		// curent cargo load (TTD: 16bit, TTO: 8bit)
		ttd_dec[ttd_off + 0x3C] = tto_dec[tto_off + 0x38];
		ttd_dec[ttd_off + 0x3D] = 0;

		// various (2 bytes)
		memcpy(&ttd_dec[ttd_off + 0x3E], &tto_dec[tto_off + 0x39], 2);

		// various (6 bytes)
		memcpy(&ttd_dec[ttd_off + 0x40], &tto_dec[tto_off + 0x3C], 6);

		// vehicle type ID (TTD: 16bit, TTO: 8bit)
		vehid_tto = tto_dec[tto_off + 0x42]; // read vehicle type ID
		memcpy(&ttd_dec[ttd_off + 0x46], &vehid_ttd[vehid_tto], 2); // write adjusted vehicle type ID

		// sprite set offset (1 byte)
		spriteset = tto_dec[tto_off + 0x43]; // read sprite set offset
		if (tto_dec[tto_off] == 32 || tto_dec[tto_off] == 37) // railway vehicle
		{
			spriteset = spriteset_rail[spriteset / 2]; // adjust railway sprite set offset
		}
		else if (tto_dec[tto_off] == 33) // road vehicle
		{
			if (spriteset >= 22) spriteset += 12;
		}
		else if (tto_dec[tto_off] == 34) // ship (for aircraft the value stays the same)
		{
			spriteset += 2;
			if (spriteset == 2 && tto_dec[tto_off + 0x36] != 3)
				// oil tanker && cargo type != oil
				spriteset = 0; // make it a coal/goods ship
			else if (spriteset == 4 && tto_dec[tto_off + 0x36] == 2)
				// passenger ship && cargo type == mail
				spriteset = 0; // make it a mail ship
		}
		ttd_dec[ttd_off + 0x48] = spriteset; // write adjusted sprite set offset

		// various (5 bytes; byte at offset 0x49 in TTO is unknown)
		memcpy(&ttd_dec[ttd_off + 0x49], &tto_dec[tto_off + 0x44], 5);

		// various (0x1C bytes)
		memcpy(&ttd_dec[ttd_off + 0x4E], &tto_dec[tto_off + 0x4A], 0x1C);
		if (tto_dec[tto_off])
		{
			// adjust index of next vehicle in consist
			memcpy(&veh_index, &tto_dec[tto_off + 0x56], 2);		
			if (veh_index >= 0x0280 && veh_index != 0xFFFF) veh_index += 50;
			memcpy(&ttd_dec[ttd_off + 0x5A], &veh_index, 2);

			// adjust vehicle name
			memcpy(&text_id, &tto_dec[tto_off + 0x5C], 2);
			if		(text_id == 0x8495) text_id = 0x8864;  // "Train X"
			else if (text_id == 0x8842) text_id = 0x902B;  // "Road Vehicle X"
			else if (text_id == 0x8C3B) text_id = 0x9830;  // "Ship X"
			else if	(text_id == 0x9047) text_id = 0xA02F;  // "Aircraft X"
			else if	(text_id != 0x0006) text_id += 0x2A2C; // custom name
			memcpy(&ttd_dec[ttd_off + 0x60], &text_id, 2);

			// adjust aircraft movement state & large airport blocks
			if (tto_dec[tto_off] == 35 && tto_dec[tto_off + 1] <= 2) // if aircraft
			{
				air_pos = tto_dec[tto_off + 0x5E];

				// adjust aircraft position in any case:
				if (air_pos >= 10) air_pos += 3;
				else if (air_pos == 9) air_pos = 11;

				// convert aircraft position to states 9 & 10 (introduced in TTD),
				// if necessary, and convert large airport blocks:
				target_airport = &ttd_dec[0x48CBA + 0x8E * tto_dec[tto_off + 0x5F]];
				if (*(target_airport + 0x81) == 1) // if type of target airport = large
				{
					air_state = tto_dec[tto_off + 0x62];

					memcpy(&air_x, &tto_dec[tto_off + 0x18], 2); // X position of the aircraft
					memcpy(&air_y, &tto_dec[tto_off + 0x1A], 2); // Y position of the aircraft
					// substract X/Y coords of the north tile of airport to get X/Y offsets:
					air_x -= 16 * *(target_airport + 0xC);
					air_y -= 16 * *(target_airport + 0xD);

					if ((air_pos == 1 && air_state == 17
					  || air_pos == 6 && (air_state == 8 || air_state == 9)
					  || air_pos == 7 && air_state == 7)
					  && air_x <= 37 && air_y >= 69)
					{
						// landing, heading for hangar/terminal and not having reached crossing
						air_pos = 9;
						*(target_airport + 0x86) &= 0x7F; // set airport to idle (bit 7)
						*(target_airport + 0x87) |= 0x04; // set in way to busy (bit 10)
						if (air_state != 17) // if going to terminal, set target terminal to occuppied
							*(target_airport + 0x86) |= (1 << (air_state - 7));
					}
					else if (air_pos == 11 && air_x >= 56 && air_y >= 69)
					{
						// moving from hangar/terminal to runway and having passed crossing
						air_pos = 10;
						*(target_airport + 0x86) &= 0x7F; // set airport to idle (bit 7)
						*(target_airport + 0x87) |= 0x08; // set runway to busy (bit 11)
					}
					else if (air_pos == 8 || air_pos == 13 || air_pos == 14 || air_pos == 21 || air_pos == 22)
					{
						// descending, braking on runway, turning towards in way,
						// entering runway or accelerating on runway
						*(target_airport + 0x86) &= 0x7F; // set airport to idle (bit 7)
						*(target_airport + 0x87) |= 0x08; // set runway to busy (bit 11)
						 // if going to terminal, set target terminal to occuppied:
						if (air_pos != 13 && air_pos != 14 && air_state != 17)
							*(target_airport + 0x86) |= (1 << (air_state - 7));
					}
				}
				ttd_dec[ttd_off + 0x62] = air_pos; // write adjusted position
			}
			else
			{
				memcpy(&sprite, &tto_dec[tto_off + 0x5E], 2);		
				if (sprite >= 2520 && sprite <= 2522) // if zeppelin
				{
					sprite += 1385; // adjust sprite (= movement state for zeppelins)
					memcpy(&ttd_dec[ttd_off + 0x62], &sprite, 2);
				}
			}

			// adjust vehicle type
			if (tto_dec[tto_off] == 32) // conventional railway
				ttd_dec[ttd_off + 0x66] = 0;
			else if (tto_dec[tto_off] == 37) // monorail
				ttd_dec[ttd_off + 0x66] = 1;
		}

		// collision detection counter (1 byte, not stored in TTO, used for vehicle reversing)
		ttd_dec[ttd_off + 0x6A] = 0;

		// 0x15 bytes with unknown meaning (fillers?)
		memset(&ttd_dec[ttd_off + 0x6B], 0, 0x15);

		tto_off += 0x80;
		ttd_off += 0x80;
	}

	// custom strings array (TTD: 500 entries, 0x20 bytes each)
	tto_off = 0x45746;
	ttd_off = 0x6F0F2;
	memset(&ttd_dec[ttd_off], 0, 300*0x20); // write 300 empty entries (TTO: only 200)
	ttd_off += 300*0x20;
	for (i = 0; i < 200; ++i) // copy all 200 TTO entries
	{
		memcpy(&ttd_dec[ttd_off + 0x20*i], &tto_dec[tto_off + 0x18*i], 0x18);
		memset(&ttd_dec[ttd_off + 0x20*i + 0x18], 0, 8); // TTD entries are 8 bytes longer
	}

	// vehicle bounding blocks list (0x1000 entries, 2 bytes each)
	tto_off = 0x46A06;
	ttd_off = 0x72F72;
	for (i = 0; i < 0x1000; ++i)
	{
		memcpy(&veh_index, &tto_dec[tto_off + 2*i], 2);
		if (veh_index != 0xFFFF && veh_index >= 640) // special vehicle?
			veh_index += 50;
		memcpy(&ttd_dec[ttd_off + 2*i], &veh_index, 2);
	}

	// signs array (40 entries, 14 bytes each)
	tto_off = 0x48A06;
	ttd_off = 0x74F72;
	for (i = 0; i < 40; ++i)
	{
		memcpy(&text_id, &tto_dec[tto_off], 2);
		if (text_id == 0x140A) // sign text: "Sign"
			text_id = 0x280A;
		else if (text_id) // custom sign text
			text_id += 10540;
		memcpy(&ttd_dec[ttd_off], &text_id, 2);
		memcpy(&ttd_dec[ttd_off + 2], &tto_dec[tto_off + 2], 12);
		if (text_id)
		{
			// adjust text width
			ttd_dec[ttd_off + 8] -= 2;
			ttd_dec[ttd_off + 9] += 2;
		}
		tto_off += 14;
		ttd_off += 14;
	}

	// Vehicle types array (TTD: 256 entries, 0x1C bytes each)
	tto_off = 0x48C36;
	ttd_off = 0x751A2;
	const unsigned char vehtype_id [256] = { // see separate documentation
		0,	255,255,255,255,255,255,255,5,	7,	8,	9,	10,	11,	12,	13,
		255,255,255,255,255,255,15,	16,	17,	18,	19,	20,	21,	22,	23,	24,
		25,	26,	27,	29,	28,	30,	255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,254,31,	255,32,	33,	34,	35,	36,	37,	38,
		39,	40,	41,	42,	255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,254,254,254,254,255,254,254,254,254,254,254,254,
		254,254,254,254,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,44,	45,	46,	254,255,255,255,47,	48,	254,49,	50,
		254,255,255,255,51,	52,	254,53,	54,	254,55,	56,	254,57,	59,	254,
		58,	60,	254,61,	62,	254,63,	64,	254,65,	66,	254,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
		255,255,255,255,255,255,255,255,255,255,255,255,67,	68,	69,	70,
		71,	255,255,76,	77,	255,255,78,	79,	80,	81,	82,	83,	84,	85,	86,
		87,	88,	89,	90,	91,	92,	93,	94,	95,	96,	97,	98,	99,	100,101,254,
		254,254,254,254,254,254,254,254,255,255,255,255,255,102,254,255};
	const unsigned short vt_data [256][10] = { // vehicle type array data:
		// introduction date, new reliability, reliability decrease, initial reliability,
		// maximum reliability, final reliability, duration phase 1, 2, 3, life span
		{0x1443, 0xB9C7, 0x0050, 0xB9C7, 0xF9E4, 0x5217, 0x0025, 0x010B, 0x00D2, 0x0F},
		{0x3398, 0x8563, 0x0050, 0x8563, 0xED73, 0x73EC, 0x0009, 0x010C, 0x00C8, 0x16},
		{0x26FD, 0x8EF3, 0x0050, 0x8EF3, 0xC414, 0x600A, 0x0011, 0x0204, 0x00C7, 0x14},
		{0x2F9D, 0x99DE, 0x0050, 0x99DE, 0xCB37, 0x6B12, 0x0012, 0x010E, 0x00DA, 0x14},
		{0x423F, 0x9F7C, 0x0050, 0x9F7C, 0xFCF1, 0x4C0F, 0x0007, 0x0115, 0x00B3, 0x14},
		{0x4AD8, 0x9AD3, 0x0050, 0x9AD3, 0xDEF0, 0x5108, 0x0009, 0x0115, 0x00AC, 0x14},
		{0x51D9, 0xB4DB, 0x0050, 0xB4DB, 0xCCF2, 0x6CD0, 0x0016, 0x0114, 0x008E, 0x14},
		{0x2259, 0x95B2, 0x0050, 0x95B2, 0xD005, 0x5520, 0x001D, 0x010B, 0x007F, 0x14},
		{0x14E9, 0xA1FC, 0x0050, 0xA1FC, 0xE564, 0x6553, 0x000C, 0x0109, 0x00AA, 0x15},
		{0x1697, 0xA1C6, 0x0050, 0xA1C6, 0xF824, 0x5A35, 0x0014, 0x010C, 0x00D1, 0x14},
		{0x31FA, 0xB43D, 0x0050, 0xB43D, 0xD615, 0x62F1, 0x0017, 0x00D9, 0x0088, 0x17},
		{0x33E6, 0x9E57, 0x0050, 0x9E57, 0xD3F5, 0x7F52, 0x001D, 0x0117, 0x00BF, 0x0C},
		{0x5CE4, 0x7CCE, 0x0050, 0x7CCE, 0xFD4D, 0x7028, 0x000C, 0x0152, 0x00E8, 0x0F},
		{0x3A98, 0x90FD, 0x0050, 0x90FD, 0xDAE3, 0x59EE, 0x0012, 0x00F0, 0x0096, 0x12},
		{0x38DF, 0xA283, 0x0050, 0xA283, 0xF70C, 0x7FEC, 0x0026, 0x0113, 0x00EF, 0x14},
		{0x3D37, 0x88EF, 0x0050, 0x88EF, 0xF7FA, 0x502D, 0x0011, 0x0131, 0x00CA, 0x16},
		{0x3C47, 0x86C9, 0x0050, 0x86C9, 0xCA5C, 0x5764, 0x0007, 0x00D7, 0x008F, 0x14},
		{0x4042, 0x98EA, 0x0050, 0x98EA, 0xF82C, 0x5838, 0x0012, 0x0114, 0x00DF, 0x14},
		{0x4BBC, 0xA981, 0x0050, 0xA981, 0xD851, 0x74A6, 0x000E, 0x010E, 0x0083, 0x16},
		{0x376D, 0xAE31, 0x0050, 0xAE31, 0xE072, 0x5054, 0x000C, 0x0115, 0x00AA, 0x16},
		{0x5094, 0xA782, 0x0050, 0xA782, 0xEF2E, 0x6016, 0x0014, 0x010F, 0x00E3, 0x16},
		{0x3F73, 0xA825, 0x0050, 0xA825, 0xEC8E, 0x526B, 0x000D, 0x0114, 0x009C, 0x16},
		{0x5290, 0x7C69, 0x0050, 0x7C69, 0xE34D, 0x587C, 0x0023, 0x00CE, 0x0091, 0x14},
		{0x40FF, 0xA99B, 0x0050, 0xA99B, 0xF326, 0x686C, 0x0019, 0x0115, 0x00F4, 0x17},
		{0x4D6B, 0x826D, 0x0050, 0x826D, 0xEFCA, 0x4854, 0x000F, 0x036A, 0x009E, 0x17},
		{0x5BA8, 0x8AEA, 0x0050, 0x8AEA, 0xFFAF, 0x667D, 0x0021, 0x0117, 0x007D, 0x19},
		{0x67B5, 0x88C5, 0x0050, 0x88C5, 0xCF37, 0x4C79, 0x001B, 0x01FA, 0x00B2, 0x19},
		{0x0802, 0xEC5F, 0x0050, 0x953A, 0xEC5F, 0x4BB4, 0x0021, 0x01FF, 0x00F1, 0x14},
		{0x0880, 0xC3FC, 0x0050, 0x97F6, 0xC3FC, 0x5EA0, 0x0016, 0x01FC, 0x007D, 0x14},
		{0x074E, 0xCB13, 0x0050, 0x96DD, 0xCB13, 0x5A89, 0x0021, 0x0205, 0x007A, 0x14},
		{0x087F, 0xF62A, 0x0050, 0x8F06, 0xF62A, 0x536B, 0x0018, 0x0206, 0x00F1, 0x14},
		{0x08A7, 0xFE25, 0x0050, 0xB812, 0xFE25, 0x5EE2, 0x001C, 0x0206, 0x0087, 0x14},
		{0x075D, 0xCDD2, 0x0050, 0x81DE, 0xCDD2, 0x6336, 0x0013, 0x0205, 0x00CB, 0x14},
		{0x08CC, 0xDAED, 0x0050, 0x7F62, 0xDAED, 0x4FA5, 0x0024, 0x0204, 0x0098, 0x14},
		{0x085A, 0xD905, 0x0050, 0x9D11, 0xD905, 0x7A07, 0x0026, 0x01FC, 0x00EB, 0x14},
		{0x08A9, 0xEC25, 0x0050, 0xADCD, 0xEC25, 0x761A, 0x0012, 0x01FC, 0x00F1, 0x14},
		{0x07B9, 0xD4F3, 0x0050, 0xA2CE, 0xD4F3, 0x69A4, 0x001F, 0x01FA, 0x00E0, 0x14},
		{0x08C3, 0xFAED, 0x0050, 0x7BC6, 0xFAED, 0x66D3, 0x001E, 0x0200, 0x00AA, 0x14},
		{0x0803, 0xC37C, 0x0050, 0x97E9, 0xC37C, 0x7064, 0x0025, 0x0201, 0x00AB, 0x14},
		{0x0797, 0xF7DF, 0x0050, 0xA414, 0xF7DF, 0x7C56, 0x001C, 0x01FD, 0x00E6, 0x14},
		{0x087C, 0xCACA, 0x0050, 0x7CDC, 0xCACA, 0x6651, 0x0021, 0x01FF, 0x0079, 0x14},
		{0x087E, 0xE75B, 0x0050, 0xB94C, 0xE75B, 0x5DFF, 0x0010, 0x0203, 0x00BD, 0x14},
		{0x07E3, 0xC166, 0x0050, 0xA874, 0xC166, 0x5C3A, 0x0024, 0x0203, 0x00EB, 0x14},
		{0x0866, 0xFBF0, 0x0050, 0xADCE, 0xFBF0, 0x7605, 0x0024, 0x0206, 0x0081, 0x14},
		{0x08E5, 0xCABB, 0x0050, 0x8455, 0xCABB, 0x46FB, 0x001A, 0x01FB, 0x00B2, 0x14},
		{0x0830, 0xF613, 0x0050, 0x9501, 0xF613, 0x4B75, 0x000A, 0x0200, 0x0093, 0x14},
		{0x0879, 0xCE2C, 0x0050, 0x84F5, 0xCE2C, 0x4338, 0x0024, 0x0207, 0x007B, 0x14},
		{0x0806, 0xDCF5, 0x0050, 0xB5CC, 0xDCF5, 0x7A0F, 0x0013, 0x0202, 0x0096, 0x14},
		{0x08C3, 0xC3A1, 0x0050, 0x9B59, 0xC3A1, 0x78F4, 0x001C, 0x01FA, 0x00E2, 0x14},
		{0x081D, 0xCB1D, 0x0050, 0x858D, 0xCB1D, 0x500B, 0x001A, 0x01FA, 0x00A2, 0x14},
		{0x0867, 0xC6FD, 0x0050, 0x7FDB, 0xC6FD, 0x7A69, 0x0017, 0x01F8, 0x009C, 0x14},
		{0x08E2, 0xEB40, 0x0050, 0xA899, 0xEB40, 0x44AF, 0x0007, 0x01F8, 0x00B7, 0x14},
		{0x07F8, 0xF153, 0x0050, 0x972F, 0xF153, 0x6C7B, 0x0008, 0x01F8, 0x00C3, 0x14},
		{0x08A0, 0xF099, 0x0050, 0x9491, 0xF099, 0x46D4, 0x000E, 0x0200, 0x00A8, 0x14},
		{0x707C, 0x8192, 0x0050, 0x8192, 0xCFA8, 0x5AAE, 0x001D, 0x01FC, 0x00C4, 0x14},
		{0x79CD, 0xA91D, 0x0050, 0xA91D, 0xD529, 0x7B39, 0x0013, 0x0201, 0x0085, 0x14},
		{0x7104, 0xA6C6, 0x0050, 0xA6C6, 0xF440, 0x688C, 0x001D, 0x0204, 0x00E6, 0x14},
		{0x08B1, 0xF12D, 0x0050, 0x8204, 0xF12D, 0x4B91, 0x000E, 0x01FF, 0x00E9, 0x14},
		{0x090C, 0xF27B, 0x0050, 0x9D4B, 0xF27B, 0x45F6, 0x0014, 0x01FB, 0x00BE, 0x14},
		{0x0830, 0xF067, 0x0050, 0xABF7, 0xF067, 0x7F76, 0x0022, 0x0207, 0x00AC, 0x14},
		{0x0843, 0xCCE6, 0x0050, 0xA5F6, 0xCCE6, 0x7E12, 0x0012, 0x01FA, 0x00EA, 0x14},
		{0x0825, 0xF354, 0x0050, 0xB426, 0xF354, 0x7C88, 0x001A, 0x01FC, 0x00E5, 0x14},
		{0x08C6, 0xF4BC, 0x0050, 0x84AB, 0xF4BC, 0x4E57, 0x000C, 0x0207, 0x00C5, 0x14},
		{0x07D9, 0xE38B, 0x0050, 0xA7A4, 0xE38B, 0x5974, 0x0020, 0x01FD, 0x009F, 0x14},
		{0x07BC, 0xE7A2, 0x0050, 0xB566, 0xE7A2, 0x709C, 0x0019, 0x01FF, 0x00C7, 0x14},
		{0x08D2, 0xF4B8, 0x0050, 0xA7A6, 0xF4B8, 0x58D2, 0x0014, 0x01FB, 0x00AD, 0x14},
		{0x0832, 0xD97D, 0x0050, 0x7E25, 0xD97D, 0x55F7, 0x0021, 0x0205, 0x0096, 0x14},
		{0x089A, 0xE255, 0x0050, 0xAECF, 0xE255, 0x6C3E, 0x0017, 0x01FF, 0x00F5, 0x14},
		{0x0803, 0xF5A3, 0x0050, 0xA776, 0xF5A3, 0x4D20, 0x0016, 0x01F8, 0x0080, 0x14},
		{0x07CD, 0xDBE7, 0x0050, 0x7B12, 0xDBE7, 0x7E27, 0x001E, 0x0205, 0x0093, 0x14},
		{0x0800, 0xE5E9, 0x0050, 0xB678, 0xE5E9, 0x54A2, 0x0017, 0x0206, 0x0085, 0x14},
		{0x08C2, 0xF496, 0x0050, 0x998B, 0xF496, 0x5B4E, 0x0010, 0x01FA, 0x007A, 0x14},
		{0x0847, 0xFD7F, 0x0050, 0xA5FD, 0xFD7F, 0x4A64, 0x0018, 0x01FB, 0x00B4, 0x14},
		{0x0744, 0xC182, 0x0050, 0x7FAE, 0xC182, 0x55BA, 0x0022, 0x01F9, 0x00A9, 0x14},
		{0x0899, 0xD752, 0x0050, 0xB86B, 0xD752, 0x7129, 0x0010, 0x01FE, 0x00D1, 0x14},
		{0x08B0, 0xD652, 0x0050, 0xAF27, 0xD652, 0x5DE1, 0x0014, 0x0204, 0x00E5, 0x14},
		{0x0819, 0xD405, 0x0050, 0xAD2A, 0xD405, 0x56D7, 0x001D, 0x01F8, 0x00DE, 0x14},
		{0x0743, 0xC18D, 0x0050, 0x849C, 0xC18D, 0x7025, 0x000E, 0x0204, 0x00D5, 0x14},
		{0x07AD, 0xDED9, 0x0050, 0xAD42, 0xDED9, 0x7B07, 0x0023, 0x01F8, 0x00F6, 0x14},
		{0x07AA, 0xEC2B, 0x0050, 0x9D9D, 0xEC2B, 0x6329, 0x001D, 0x01FE, 0x007C, 0x14},
		{0x07DD, 0xE059, 0x0050, 0x9337, 0xE059, 0x5737, 0x0011, 0x0203, 0x0080, 0x14},
		{0x0786, 0xDA6D, 0x0050, 0xBA4B, 0xDA6D, 0x5D33, 0x0015, 0x0207, 0x00D9, 0x14},
		{0x084D, 0xC302, 0x0050, 0x8FF8, 0xC302, 0x5C9E, 0x0011, 0x01F9, 0x0098, 0x14},
		{0x0789, 0xE290, 0x0050, 0x8135, 0xE290, 0x5E15, 0x0026, 0x0203, 0x00B4, 0x14},
		{0x9065, 0x7B24, 0x0050, 0x7B24, 0xEE5C, 0x713F, 0x0026, 0x01FD, 0x0085, 0x14},
		{0x9A1B, 0x8EB2, 0x0050, 0x8EB2, 0xDE90, 0x7088, 0x0010, 0x0201, 0x0093, 0x14},
		{0xA523, 0x9391, 0x0050, 0x9391, 0xE892, 0x71E6, 0x000D, 0x0202, 0x00EE, 0x14},
		{0xA79C, 0x8283, 0x0050, 0x8283, 0xF876, 0x6A09, 0x0007, 0x0207, 0x00D4, 0x14},
		{0x90E9, 0x8BB0, 0x0050, 0x8BB0, 0xCF4C, 0x79A4, 0x0014, 0x0276, 0x00DC, 0x14},
		{0x088E, 0xCAC9, 0x0050, 0x9217, 0xCAC9, 0x7DF6, 0x0023, 0x0200, 0x00DC, 0x14},
		{0x08BE, 0xDBA4, 0x0050, 0xA440, 0xDBA4, 0x7020, 0x0011, 0x01F8, 0x0085, 0x14},
		{0x084A, 0xC65E, 0x0050, 0x882F, 0xC65E, 0x49FA, 0x0017, 0x01FA, 0x009C, 0x14},
		{0x07F4, 0xEA34, 0x0050, 0x8EEE, 0xEA34, 0x4AF3, 0x0009, 0x01FE, 0x00E9, 0x14},
		{0x089F, 0xE3BB, 0x0050, 0x9335, 0xE3BB, 0x7BE3, 0x0024, 0x01FF, 0x00D2, 0x14},
		{0x0858, 0xCAAD, 0x0050, 0x811D, 0xCAAD, 0x4FA4, 0x0025, 0x0202, 0x00BD, 0x14},
		{0x07EF, 0xE244, 0x0050, 0x91C5, 0xE244, 0x457D, 0x0008, 0x01F8, 0x00E6, 0x14},
		{0x07A1, 0xF6ED, 0x0050, 0x8B6E, 0xF6ED, 0x54E9, 0x0009, 0x0203, 0x00B2, 0x14},
		{0x082A, 0xE759, 0x0050, 0x93CE, 0xE759, 0x4A48, 0x000B, 0x0204, 0x00E9, 0x14},
		{0x08C6, 0xFD5C, 0x0050, 0x8B79, 0xFD5C, 0x5419, 0x0025, 0x0202, 0x00B0, 0x14},
		{0x090B, 0xFDF0, 0x0050, 0x86FA, 0xFDF0, 0x5E12, 0x0012, 0x0202, 0x007E, 0x14},
		{0x0851, 0xD98E, 0x0050, 0xB77A, 0xD98E, 0x5AEE, 0x001A, 0x0203, 0x007F, 0x14},
		{0x08EF, 0xDA15, 0x0050, 0xA174, 0xDA15, 0x64C2, 0x0018, 0x01F8, 0x009C, 0x14},
		{0x084C, 0xEFBD, 0x0050, 0x928E, 0xEFBD, 0x4E85, 0x0012, 0x01FF, 0x00F1, 0x14},
		{0x08FC, 0xFAAE, 0x0050, 0xB494, 0xFAAE, 0x5B6D, 0x000D, 0x0201, 0x0079, 0x14},
		{0x07F7, 0xC09D, 0x0050, 0xBA69, 0xC09D, 0x5F2B, 0x0021, 0x01FF, 0x0087, 0x14},
		{0x0801, 0xD295, 0x0050, 0x9EA8, 0xD295, 0x7FFB, 0x0017, 0x0201, 0x00E9, 0x14},
		{0x0856, 0xCB57, 0x0050, 0x894B, 0xCB57, 0x7206, 0x000C, 0x0206, 0x0095, 0x14},
		{0x07FD, 0xC792, 0x0050, 0xA0A4, 0xC792, 0x7E52, 0x0026, 0x01F9, 0x007B, 0x14},
		{0x080B, 0xF50A, 0x0050, 0x95D2, 0xF50A, 0x7A0D, 0x000C, 0x01F8, 0x00A6, 0x14},
		{0x078D, 0xCA22, 0x0050, 0xAEBF, 0xCA22, 0x6842, 0x000F, 0x0203, 0x00B7, 0x14},
		{0x07D7, 0xCA50, 0x0050, 0x858A, 0xCA50, 0x7AC6, 0x000F, 0x0205, 0x00C7, 0x14},
		{0x085C, 0xE856, 0x0050, 0xB9D0, 0xE856, 0x4B01, 0x0011, 0x0206, 0x0091, 0x14},
		{0x08D0, 0xF9B8, 0x0050, 0x86C8, 0xF9B8, 0x458C, 0x0011, 0x01FB, 0x009E, 0x14},
		{0x0842, 0xE491, 0x0050, 0xB403, 0xE491, 0x6F79, 0x0015, 0x01FC, 0x00A3, 0x14},
		{0x077D, 0xEA73, 0x0050, 0xBA61, 0xEA73, 0x74A4, 0x000D, 0x01F9, 0x00CD, 0x14},
		{0x08B9, 0xF93F, 0x0050, 0x8246, 0xF93F, 0x5211, 0x001C, 0x0200, 0x0089, 0x14},
		{0x0DDA, 0x9037, 0x0050, 0x81B7, 0xF5B7, 0x6262, 0x0018, 0x0188, 0x0082, 0x0C},
		{0x405F, 0x9887, 0x0050, 0x9887, 0xD7CC, 0x7B04, 0x000B, 0x0110, 0x00EC, 0x0F},
		{0x5E4A, 0x9335, 0x0050, 0x9335, 0xD1E6, 0x7B50, 0x0023, 0x0184, 0x0096, 0x0F},
		{0x7F3B, 0x8547, 0x0050, 0x8547, 0xE651, 0x62C8, 0x000B, 0x036F, 0x0088, 0x0F},
		{0x2527, 0x949D, 0x0050, 0x949D, 0xC795, 0x7590, 0x000F, 0x0184, 0x00E4, 0x0F},
		{0x4BDC, 0x89D7, 0x0050, 0x89D7, 0xFE5C, 0x53E7, 0x001D, 0x018F, 0x00D8, 0x0F},
		{0x81C7, 0xB6DE, 0x0050, 0xB6DE, 0xD679, 0x63D6, 0x000B, 0x0362, 0x007C, 0x0F},
		{0x16F3, 0x9F6E, 0x0050, 0x9F6E, 0xC630, 0x636D, 0x001A, 0x023F, 0x00AF, 0x0F},
		{0x4F99, 0xA1CC, 0x0050, 0xA1CC, 0xCE26, 0x790C, 0x0014, 0x023E, 0x008B, 0x0F},
		{0x84F9, 0xAE7E, 0x0050, 0xAE7E, 0xFE88, 0x4ECA, 0x0026, 0x039F, 0x00F1, 0x0F},
		{0x1660, 0xAD1E, 0x0050, 0xAD1E, 0xDAA7, 0x53D7, 0x0020, 0x0234, 0x0085, 0x0F},
		{0x55ED, 0x8FEA, 0x0050, 0x8FEA, 0xC712, 0x7190, 0x0009, 0x0237, 0x007E, 0x0F},
		{0x8DA9, 0xA710, 0x0050, 0xA710, 0xF7EB, 0x6A93, 0x001E, 0x03A0, 0x00C7, 0x0F},
		{0x16D2, 0xAEB4, 0x0050, 0xAEB4, 0xE6F7, 0x7FEE, 0x001E, 0x023B, 0x00EB, 0x0F},
		{0x5514, 0x7C00, 0x0050, 0x7C00, 0xF8B5, 0x6454, 0x0022, 0x023E, 0x00BE, 0x0F},
		{0x8BD4, 0x80FE, 0x0050, 0x80FE, 0xCD00, 0x5489, 0x001D, 0x03AB, 0x00B5, 0x0F},
		{0x1764, 0xA112, 0x0050, 0xA112, 0xC750, 0x76C2, 0x000F, 0x0234, 0x00D5, 0x0F},
		{0x4D93, 0x7BFA, 0x0050, 0x7BFA, 0xDF37, 0x699D, 0x0010, 0x0237, 0x0083, 0x0F},
		{0x7AD1, 0x9F0A, 0x0050, 0x9F0A, 0xE93A, 0x414C, 0x001C, 0x03A6, 0x009D, 0x0F},
		{0x1636, 0x957A, 0x0050, 0x957A, 0xEDB4, 0x5149, 0x0025, 0x0240, 0x00E5, 0x0F},
		{0x56BC, 0x9349, 0x0050, 0x9349, 0xC56B, 0x7D0F, 0x0015, 0x0237, 0x00CA, 0x0F},
		{0x931D, 0x88C8, 0x0050, 0x88C8, 0xF671, 0x7962, 0x0020, 0x03A7, 0x0089, 0x0F},
		{0x16F7, 0xABF8, 0x0050, 0xABF8, 0xE845, 0x4CC8, 0x001E, 0x0236, 0x009E, 0x0F},
		{0x4D4C, 0x9BF8, 0x0050, 0x9BF8, 0xC7A8, 0x5790, 0x0020, 0x0242, 0x00B6, 0x0F},
		{0x7A36, 0x8185, 0x0050, 0x8185, 0xCDD8, 0x56F0, 0x0020, 0x03A2, 0x00E8, 0x0F},
		{0x1570, 0x9951, 0x0050, 0x9951, 0xD0D6, 0x4266, 0x0017, 0x023E, 0x008A, 0x0F},
		{0x5360, 0xB0E3, 0x0050, 0xB0E3, 0xCD26, 0x7150, 0x0023, 0x023E, 0x00CB, 0x0F},
		{0x824F, 0x9B39, 0x0050, 0x9B39, 0xE18D, 0x6B6C, 0x001C, 0x03A5, 0x00CB, 0x0F},
		{0x1634, 0x8BB9, 0x0050, 0x8BB9, 0xF1DE, 0x5BE7, 0x0017, 0x023C, 0x00C2, 0x0F},
		{0x4E84, 0x7F4A, 0x0050, 0x7F4A, 0xFB44, 0x51A9, 0x0022, 0x0234, 0x008E, 0x0F},
		{0x8BFD, 0xA631, 0x0050, 0xA631, 0xE0F2, 0x45C4, 0x000C, 0x039F, 0x00DB, 0x0F},
		{0x1635, 0x8486, 0x0050, 0x8486, 0xD435, 0x70A9, 0x0011, 0x0235, 0x0083, 0x0F},
		{0x515A, 0xAF8E, 0x0050, 0xAF8E, 0xD593, 0x51C4, 0x000C, 0x0242, 0x00DE, 0x0F},
		{0x82E6, 0x9CA0, 0x0050, 0x9CA0, 0xC5FB, 0x681D, 0x000E, 0x03A7, 0x00A4, 0x0F},
		{0x16F8, 0x7E07, 0x0050, 0x7E07, 0xCD17, 0x780C, 0x0023, 0x0235, 0x00EB, 0x0F},
		{0x5496, 0xA96C, 0x0050, 0xA96C, 0xC3E1, 0x71A3, 0x000C, 0x0237, 0x00A2, 0x0F},
		{0x7C34, 0x9D20, 0x0050, 0x9D20, 0xC232, 0x77BB, 0x001E, 0x03A4, 0x00D4, 0x0F},
		{0x15E5, 0xB32A, 0x0050, 0xB32A, 0xF603, 0x6126, 0x0013, 0x0236, 0x00CC, 0x0F},
		{0x58EE, 0x984B, 0x0050, 0x984B, 0xDA87, 0x4867, 0x0015, 0x0240, 0x00E9, 0x0F},
		{0x84CD, 0x8C42, 0x0050, 0x8C42, 0xDD31, 0x54F8, 0x000A, 0x03A6, 0x009E, 0x0F},
		{0x1746, 0x8F84, 0x0050, 0x8F84, 0xF3EA, 0x4AF1, 0x0007, 0x0238, 0x00A1, 0x0F},
		{0x4A81, 0x8AC2, 0x0050, 0x8AC2, 0xE143, 0x6584, 0x001C, 0x0240, 0x00CB, 0x0F},
		{0x795C, 0xA29B, 0x0050, 0xA29B, 0xE137, 0x72E0, 0x0011, 0x03AA, 0x00F0, 0x0F},
		{0x1684, 0x9457, 0x0050, 0x9457, 0xD5C7, 0x5B5B, 0x0007, 0x0243, 0x00C8, 0x0F},
		{0x545E, 0x9462, 0x0050, 0x9462, 0xF501, 0x657E, 0x0022, 0x0236, 0x00EB, 0x0F},
		{0x7D31, 0xB153, 0x0050, 0xB153, 0xD9F8, 0x79A5, 0x0015, 0x03AB, 0x00EA, 0x0F},
		{0x16C9, 0x9EA4, 0x0050, 0x9EA4, 0xDB02, 0x6735, 0x0025, 0x0236, 0x00CA, 0x0F},
		{0x52D6, 0xB90F, 0x0050, 0xB90F, 0xEC61, 0x497D, 0x001E, 0x023D, 0x00DE, 0x0F},
		{0x8325, 0x9D5F, 0x0050, 0x9D5F, 0xDA73, 0x48C7, 0x0011, 0x03A0, 0x00F5, 0x0F},
		{0x1602, 0x97DC, 0x0050, 0x97DC, 0xC449, 0x5D7D, 0x000E, 0x0235, 0x0094, 0x0F},
		{0x527E, 0x7D1E, 0x0050, 0x7D1E, 0xF70B, 0x5B1B, 0x0012, 0x0234, 0x00DB, 0x0F},
		{0x836B, 0x9ACB, 0x0050, 0x9ACB, 0xCAB3, 0x6712, 0x000D, 0x03A3, 0x0085, 0x0F},
		{0x16B9, 0x8B79, 0x0050, 0x8B79, 0xCD88, 0x6D42, 0x0017, 0x0238, 0x00CF, 0x0F},
		{0x5439, 0xB4FE, 0x0050, 0xB4FE, 0xCEE1, 0x441F, 0x0011, 0x023E, 0x0099, 0x0F},
		{0x840D, 0xAB5E, 0x0050, 0xAB5E, 0xC126, 0x47FD, 0x0024, 0x03A2, 0x00BA, 0x0F},
		{0x164B, 0x9C5E, 0x0050, 0x9C5E, 0xE8E7, 0x70EE, 0x0017, 0x023F, 0x00C7, 0x0F},
		{0x50B7, 0x9C36, 0x0050, 0x9C36, 0xC273, 0x6D91, 0x000B, 0x0242, 0x0091, 0x0F},
		{0x823B, 0x9B19, 0x0050, 0x9B19, 0xC5E0, 0x59C5, 0x0016, 0x039F, 0x00EC, 0x0F},
		{0x1722, 0xB91C, 0x0050, 0xB91C, 0xF03F, 0x4103, 0x0017, 0x0243, 0x00D1, 0x0F},
		{0x4E2E, 0xA8DC, 0x0050, 0xA8DC, 0xC13B, 0x77A7, 0x0009, 0x023C, 0x00C0, 0x0F},
		{0x8337, 0xAA40, 0x0050, 0xAA40, 0xF2D3, 0x7D2C, 0x0008, 0x03A8, 0x00C2, 0x0F},
		{0x15D8, 0x8A16, 0x0050, 0x8A16, 0xF3A6, 0x5C8C, 0x0015, 0x0238, 0x009F, 0x0F},
		{0x4F49, 0xA5AD, 0x0050, 0xA5AD, 0xD20C, 0x40F5, 0x000E, 0x0238, 0x00E0, 0x0F},
		{0x845C, 0xAA58, 0x0050, 0xAA58, 0xD85E, 0x5556, 0x0011, 0x03A3, 0x00E5, 0x0F},
		{0x16B4, 0xAF70, 0x0050, 0xAF70, 0xD0A0, 0x7940, 0x000F, 0x023F, 0x0092, 0x0F},
		{0x50DB, 0xAE34, 0x0050, 0xAE34, 0xE785, 0x6C96, 0x001D, 0x0234, 0x007A, 0x0F},
		{0x864E, 0x82D7, 0x0050, 0x82D7, 0xEC8B, 0x6766, 0x0019, 0x03A8, 0x00A9, 0x0F},
		{0x15CA, 0xA31D, 0x0050, 0xA31D, 0xE3CB, 0x7BC8, 0x0023, 0x023B, 0x00F1, 0x0F},
		{0x51C9, 0x83BE, 0x0050, 0x83BE, 0xC011, 0x4A8E, 0x0022, 0x0239, 0x00E9, 0x0F},
		{0x880A, 0x9CB3, 0x0050, 0x9CB3, 0xEDDA, 0x6559, 0x0007, 0x03A3, 0x00C7, 0x0F},
		{0x1583, 0x7E3B, 0x0050, 0x7E3B, 0xDC60, 0x4F82, 0x000F, 0x023B, 0x0093, 0x0F},
		{0x53C7, 0x80B6, 0x0050, 0x80B6, 0xFDD1, 0x5EEC, 0x000F, 0x0237, 0x00AA, 0x0F},
		{0x8805, 0xA97A, 0x0050, 0xA97A, 0xFB38, 0x4218, 0x0015, 0x03A4, 0x00C1, 0x0F},
		{0x16BC, 0xB480, 0x0050, 0xB480, 0xDF5D, 0x6751, 0x0008, 0x0241, 0x00AB, 0x0F},
		{0x54D6, 0x9512, 0x0050, 0x9512, 0xDC49, 0x4D6C, 0x0020, 0x0243, 0x00C3, 0x0F},
		{0x8A16, 0x8D23, 0x0050, 0x8D23, 0xF3C6, 0x5F6A, 0x000F, 0x03A8, 0x00E9, 0x0F},
		{0x16EA, 0x7E80, 0x0050, 0x7E80, 0xE652, 0x474D, 0x0026, 0x0237, 0x00A3, 0x0F},
		{0x4F94, 0xAC95, 0x0050, 0xAC95, 0xEB30, 0x66DD, 0x0014, 0x023B, 0x00E4, 0x0F},
		{0x8BC9, 0xAD8A, 0x0050, 0xAD8A, 0xE786, 0x778C, 0x0011, 0x03A2, 0x008E, 0x0F},
		{0x1593, 0x8D2A, 0x0050, 0x8D2A, 0xECF6, 0x6CAD, 0x0019, 0x0242, 0x00B9, 0x0F},
		{0x5106, 0xB21B, 0x0050, 0xB21B, 0xDC90, 0x66CE, 0x000B, 0x023C, 0x00CE, 0x0F},
		{0x8C57, 0xA63C, 0x0050, 0xA63C, 0xD715, 0x75C9, 0x000B, 0x03A4, 0x00EC, 0x0F},
		{0x1585, 0x98B4, 0x0050, 0x98B4, 0xCDC7, 0x60CA, 0x0021, 0x0243, 0x00D9, 0x0F},
		{0x5142, 0xA347, 0x0050, 0xA347, 0xFEFA, 0x500B, 0x0008, 0x023F, 0x00C4, 0x0F},
		{0x8206, 0xAA16, 0x0050, 0xAA16, 0xCD9A, 0x5BF9, 0x001E, 0x03A5, 0x00E6, 0x0F},
		{0x169B, 0x9295, 0x0050, 0x9295, 0xD518, 0x7BFC, 0x000F, 0x0242, 0x00A1, 0x0F},
		{0x524C, 0xA948, 0x0050, 0xA948, 0xDDBD, 0x4405, 0x000C, 0x0239, 0x00E0, 0x0F},
		{0x82D0, 0x8541, 0x0050, 0x8541, 0xFDC5, 0x503B, 0x0011, 0x039E, 0x00BD, 0x0F},
		{0x0C30, 0xD466, 0x0014, 0x9A7D, 0xD466, 0x4E42, 0x000F, 0x01FE, 0x00E1, 0x1E},
		{0x4432, 0x93A9, 0x0014, 0x93A9, 0xF784, 0x5E54, 0x0017, 0x03E0, 0x00C0, 0x1E},
		{0x08E1, 0xF246, 0x0014, 0x9ADD, 0xF246, 0x4DF5, 0x001A, 0x023E, 0x00B4, 0x1E},
		{0x498C, 0x9D85, 0x0014, 0x9D85, 0xD248, 0x7379, 0x0010, 0x03DE, 0x00D2, 0x1E},
		{0x447B, 0x9F93, 0x0028, 0x9F93, 0xE5E8, 0x6E0A, 0x0013, 0x03E0, 0x00A1, 0x19},
		{0x2640, 0xB6D6, 0x0014, 0xB6D6, 0xFBAD, 0x519C, 0x0013, 0x0188, 0x00CF, 0x1E},
		{0x5226, 0x9264, 0x0014, 0x9264, 0xD925, 0x61B3, 0x0017, 0x03DE, 0x00DC, 0x1E},
		{0x0BF5, 0xBCFF, 0x0014, 0x8D19, 0xC7A4, 0x58FB, 0x0016, 0x0240, 0x009B, 0x1E},
		{0x4E66, 0xB089, 0x0014, 0xB089, 0xE234, 0x6CD9, 0x0025, 0x043C, 0x0083, 0x1E},
		{0x2686, 0xA45D, 0x0014, 0xA45D, 0xE927, 0x66BA, 0x0022, 0x01C6, 0x00E2, 0x1E},
		{0x5889, 0xB3B4, 0x0014, 0xB3B4, 0xD407, 0x7E4D, 0x000F, 0x03E1, 0x00A1, 0x1E},
		{0x0D69, 0xADCC, 0x0050, 0xA33D, 0xE61D, 0x5DE4, 0x0026, 0x0094, 0x00E3, 0x14},
		{0x27E3, 0xB193, 0x0050, 0xB193, 0xEC28, 0x5BDB, 0x001B, 0x0095, 0x00AB, 0x18},
		{0x320C, 0xA6D8, 0x0050, 0xA6D8, 0xE7A0, 0x7EFE, 0x001B, 0x009D, 0x0098, 0x12},
		{0x44F7, 0x95FB, 0x0050, 0x95FB, 0xCB59, 0x72F3, 0x001A, 0x014A, 0x00D3, 0x19},
		{0x134C, 0xA186, 0x0050, 0xA186, 0xE6AE, 0x6FB0, 0x0018, 0x0116, 0x00A5, 0x1E},
		{0x36B3, 0x876D, 0x0050, 0x876D, 0xDDB2, 0x6121, 0x0009, 0x00D5, 0x008D, 0x17},
		{0x40D0, 0x8150, 0x0050, 0x8150, 0xE9AC, 0x6104, 0x0012, 0x0112, 0x0093, 0x1A},
		{0x5439, 0x7EDB, 0x0050, 0x7EDB, 0xFF69, 0x5D03, 0x0018, 0x0110, 0x00B3, 0x19},
		{0x47BC, 0xA745, 0x0050, 0xA745, 0xFC9E, 0x7D9E, 0x0019, 0x0117, 0x00E5, 0x14},
		{0x633F, 0xBA88, 0x0050, 0xBA88, 0xE289, 0x442B, 0x0016, 0x010F, 0x00B4, 0x19},
		{0x3CAF, 0xA4C7, 0x0050, 0xA4C7, 0xF2E9, 0x5AB8, 0x001D, 0x00D4, 0x00B7, 0x16},
		{0x3CD0, 0x9C13, 0x0050, 0x9C13, 0xF241, 0x7144, 0x0014, 0x00CD, 0x007E, 0x19},
		{0x4427, 0xB583, 0x0050, 0xB583, 0xD990, 0x5920, 0x0025, 0x00D7, 0x00DA, 0x16},
		{0x4418, 0x941A, 0x0050, 0x941A, 0xEB70, 0x60AC, 0x0010, 0x0117, 0x0080, 0x19},
		{0x57D9, 0x8154, 0x0050, 0x8154, 0xD6A6, 0x6454, 0x0011, 0x014E, 0x00F2, 0x19},
		{0x57DB, 0xAD24, 0x0050, 0xAD24, 0xEF8F, 0x5397, 0x0021, 0x0108, 0x00F1, 0x19},
		{0x6A69, 0xB621, 0x0050, 0xB621, 0xE177, 0x5CC7, 0x001F, 0x010A, 0x00ED, 0x19},
		{0x47C3, 0x9B27, 0x0050, 0x9B27, 0xD019, 0x42F8, 0x000F, 0x010D, 0x00D1, 0x14},
		{0x4B09, 0x954E, 0x0050, 0x954E, 0xC844, 0x4410, 0x000A, 0x0147, 0x00C2, 0x18},
		{0x57AE, 0x7E99, 0x0050, 0x7E99, 0xC97B, 0x5134, 0x0025, 0x0111, 0x008E, 0x18},
		{0x5FFC, 0xB1E0, 0x0050, 0xB1E0, 0xFC02, 0x7154, 0x001C, 0x0116, 0x00E7, 0x18},
		{0x688B, 0xACE2, 0x0050, 0xACE2, 0xD0F4, 0x762A, 0x0011, 0x010B, 0x00C4, 0x18},
		{0x5603, 0xA71C, 0x0050, 0xA71C, 0xC219, 0x7912, 0x0015, 0x010A, 0x00EB, 0x19},
		{0x5FC3, 0x9E27, 0x0050, 0x9E27, 0xDE0E, 0x4685, 0x0018, 0x0146, 0x007B, 0x14},
		{0x73AC, 0x8304, 0x0050, 0x8304, 0xD04A, 0x674F, 0x000C, 0x0276, 0x007F, 0x19},
		{0x8AE8, 0x826E, 0x0050, 0x826E, 0xE245, 0x65AC, 0x000C, 0x010A, 0x00CA, 0x16},
		{0x9169, 0xA38B, 0x0050, 0xA38B, 0xFA2B, 0x7428, 0x000B, 0x0110, 0x00E7, 0x16},
		{0x9843, 0x85D3, 0x0050, 0x85D3, 0xFE4B, 0x4F19, 0x001B, 0x01F9, 0x00B6, 0x16},
		{0xA586, 0xA6B6, 0x0050, 0xA6B6, 0xC422, 0x5C4D, 0x0022, 0x0115, 0x00D8, 0x19},
		{0xB025, 0xB961, 0x0050, 0xB961, 0xF6DC, 0x6648, 0x0012, 0x010C, 0x00BB, 0x17},
		{0x9F3C, 0xA883, 0x0050, 0xA883, 0xF584, 0x5CDE, 0x001A, 0x0117, 0x00B4, 0x19},
		{0x84BC, 0x8560, 0x0050, 0x8560, 0xCB0E, 0x5B42, 0x0012, 0x0201, 0x009A, 0x19},
		{0x8260, 0x7F4C, 0x0050, 0x7F4C, 0xC7AC, 0x5BC2, 0x001E, 0x0279, 0x0094, 0x14},
		{0x24E4, 0xB635, 0x0050, 0xB635, 0xF6C1, 0x541B, 0x0017, 0x014A, 0x00A0, 0x14},
		{0x3428, 0x82A3, 0x0050, 0x82A3, 0xF8F1, 0x5DA9, 0x001F, 0x0153, 0x00EC, 0x14},
		{0x4368, 0xAE40, 0x0050, 0xAE40, 0xD75D, 0x7626, 0x0024, 0x0153, 0x0086, 0x14},
		{0x5397, 0x96D3, 0x0050, 0x96D3, 0xEBA5, 0x783E, 0x000D, 0x0445, 0x00CD, 0x14},
		{0x5ED0, 0xA8C7, 0x0050, 0xA8C7, 0xCBC4, 0x788D, 0x000A, 0x0452, 0x00D3, 0x14},
		{0x3517, 0x8B3F, 0x0050, 0x8B3F, 0xCD6E, 0x5969, 0x001A, 0x018A, 0x00C1, 0x14},
		{0x6FC0, 0x873F, 0x0050, 0x873F, 0xF5D2, 0x7A11, 0x001E, 0x0111, 0x00A7, 0x14},
		{0x3698, 0x97B5, 0x0050, 0x97B5, 0xFB73, 0x5110, 0x0021, 0x0449, 0x007D, 0x14}};
	unsigned char railway_type;
	unsigned short age; // vehicle age
	for (i = 0; i < 256; ++i)
	{
		if (vehtype_id[i] >= 254) // inexistent in TTO
		{
			// write bitmap of companies that can buy this vehicle
			if (date >= vt_data[i][0] && vehtype_id[i] == 254)
				memset(&ttd_dec[ttd_off + 0x1C*i], 0xFF, 2);
			else // vehicle from other climate or not yet released
				memset(&ttd_dec[ttd_off + 0x1C*i], 0, 2);

			// write date of introduction
			memcpy(&ttd_dec[ttd_off + 0x1C*i + 2], &vt_data[i][0], 2);

			// write age
			age = 0;
			if (date > vt_data[i][0])
				age = (date - vt_data[i][0]) / 30; // months since introduction
			memcpy(&ttd_dec[ttd_off + 0x1C*i + 4], &age, 2);
			
			// write values from reliability of new vehicle to life span
			memcpy(&ttd_dec[ttd_off + 0x1C*i + 6], &vt_data[i][1], 17);

			// write market availability
			ttd_dec[ttd_off + 0x1C*i + 0x17] = (date >= vt_data[i][0]);

			// write 2 exclusive testing values
			memset(&ttd_dec[ttd_off + 0x1C*i + 0x18], 0, 2);
		}
		else // copy data from TTO
		{
			memcpy(&ttd_dec[ttd_off + 0x1C*i], &tto_dec[tto_off + vehtype_id[i]*0x1A], 0x1A);
		}
		
		// additional 2 bytes
		if (i >= 54 && i <= 83)
			railway_type = 1; // monorail
		else if (i >= 84 && i <= 115)
			railway_type = 2; // maglev
		else
			railway_type = 0; // conventional railway or non-railway
		ttd_dec[ttd_off + 0x1C*i + 0x1A] = railway_type; // not stored in TTO
		ttd_dec[ttd_off + 0x1C*i + 0x1B] = 0; // unknown byte (filler?)
	}

	// copy vehicle processing counter
	tto_off = 0x496AC;
	ttd_off = 0x76DA2;
	memcpy(&ttd_dec[ttd_off], &tto_dec[tto_off], 2);

	// subsidy array
	tto_off = 0x496AE;
	ttd_off = 0x76DA4;
	unsigned char subs [4];
	for (i = 0; i < 8; ++i)
	{
		memcpy(subs, &tto_dec[tto_off + 4*i], 4);
		if (subs[1] < 12)
		{
			if (subs[0] == 0 || subs[0] == 2)
				subs[2] = towns[subs[2]];
			else
				subs[2] = inds[subs[2]];
			if (subs[0] == 0 || subs[0] == 2 || subs[0] == 5)
				subs[3] = towns[subs[3]];
			else
				subs[3] = inds[subs[3]];
		}
		memcpy(&ttd_dec[ttd_off + 4*i], subs, 4);
	}

	// copy AI founding counter
	tto_off = 0x496CE;
	ttd_off = 0x76DC4;
	memcpy(&ttd_dec[ttd_off], &tto_dec[tto_off], 2);

	// copy various values (0x12 bytes)
	tto_off = 0x496D2; // 0x496D0-0x496D1: monorail accessibility, see company array
	ttd_off = 0x76DC6;
	memcpy(&ttd_dec[ttd_off], &tto_dec[tto_off], 0x12);

	// write 5 text ID arrays and the cargo type icon array (12 32bit-ints each)
	tto_off = 0x496E4;
	ttd_off = 0x76DD8;
	const unsigned char startvalue [6] = {0x0F, 0x2F, 0x4F, 0x6F, 0x8F, 0xC9};
	for (i = 0; i < 6; ++i)
	{
		for (j = 0; j < 11; ++j)
		{
			ttd_dec[ttd_off + 24*i + j*2] = startvalue[i] + j;
			ttd_dec[ttd_off + 24*i + j*2 + 1] = 0x10 * (i == 5);
		}
		ttd_dec[ttd_off + 24*i + 22] = (i == 5 ? 2 : ttd_dec[ttd_off + 24*i + 20]-11);
		ttd_dec[ttd_off + 24*i + 23] = 0;
	}

	// write vehicle names text ID array (256 32bit entries)
	tto_off = 0x496E4; // still the same
	ttd_off = 0x76E68;
	for (i = 0; i < 256; ++i)
	{
		ttd_dec[ttd_off + 2*i] = i;
		ttd_dec[ttd_off + 2*i + 1] = 0x80;
	}

	// write table of vehicles the AI will use (7 * 12 values = 12 * 12 bytes)
	tto_off = 0x496E4; // still the same
	ttd_off = 0x77068;
	const unsigned char aivehicle [144] = {
		0x1B,0x00,0x1D,0x00,0x1C,0x00,0x1E,0x00,0x1F,0x00,0x20,0x00,
		0x21,0x00,0x22,0x00,0x23,0x00,0x24,0x00,0x25,0x00,0x26,0x00,
		0x39,0x00,0x3B,0x00,0x3A,0x00,0x3C,0x00,0x3D,0x00,0x3E,0x00,
		0x3F,0x00,0x40,0x00,0x41,0x00,0x42,0x00,0x43,0x00,0x44,0x00,
		0x59,0x00,0x5B,0x00,0x5A,0x00,0x5C,0x00,0x5D,0x00,0x5E,0x00,
		0x5F,0x00,0x60,0x00,0x61,0x00,0x62,0x00,0x63,0x00,0x64,0x00,
		0x74,0x00,0x7B,0x00,0x7E,0x00,0x84,0x00,0x87,0x00,0x8A,0x00,
		0x8D,0x00,0x90,0x00,0x93,0x00,0x96,0x00,0x99,0x00,0x9C,0x00,
		0x07,0x03,0x06,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,
		0xCE,0x00,0xD3,0x00,0xD3,0x00,0xCC,0x00,0xD3,0x00,0xD3,0x00,
		0xD7,0x00,0xD7,0x00,0xD7,0x00,0xD7,0x00,0xD7,0x00,0xD7,0x00,
		0x05,0x00,0x00,0x02,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00};
	memcpy(&ttd_dec[ttd_off], aivehicle, 144);

	// copy 8 values (various)
	tto_off = 0x496E4; // still the same
	ttd_off = 0x770F8;
	memcpy(&ttd_dec[ttd_off], &tto_dec[tto_off], 8);

	// copy color schemes of all companies
	tto_off = 0x496EC;
	ttd_off = 0x77100;
	for (i = 0; i < 8; ++i)
	{
		tto_color = tto_dec[tto_off + i]; // read TTO color
		ttd_dec[ttd_off + i] = ttd_color[tto_color]; // convert it to TTD & write it
	}

	// copy 41 bytes (various)
	tto_off = 0x496F4;
	ttd_off = 0x77108;
	memcpy(&ttd_dec[ttd_off], &tto_dec[tto_off], 41);
	ttd_dec[ttd_off + 3] %= 128; // switch off alternative graphics from World Editor
	if (ttd_dec[ttd_off + 5]) ++ttd_dec[ttd_off + 5]; // adjust town names style

	// write various (climate, industries, cargo unit weight, transit time thresholds...)
	tto_off = 0x4971D;
	ttd_off = 0x77131;
	const unsigned char various [72] = {
		0x00,0xFF,0x00,0x38,0x0C,0x0C,0x0C,0x0C,0x0C,0x0C,0x0C,0x00,0x00,0x06,0x06,0x09,
		0x09,0x03,0x03,0x03,0x12,0x12,0x04,0x04,0x02,0x02,0x05,0x05,0x05,0x05,0x05,0x05,
		0x01,0x01,0x08,0x08,0x01,0x10,0x04,0x10,0x03,0x08,0x10,0x10,0x10,0x10,0x02,0x00,
		0x00,0x07,0x14,0x19,0x04,0x05,0x04,0x0F,0x09,0x07,0x01,0x00,0x18,0xFF,0x5A,0xFF,
		0x12,0x1C,0x28,0xFF,0xFF,0xFF,0x20,0x1E};
	memcpy(&ttd_dec[ttd_off], various, 72);

	// write conversion evidence byte
	ttd_dec[0x24EB8] = 1;

	delete [] tto_dec;

	// data reformated, adjusted and completed
	//////////////////////////////////////////////////////////

	m_progressline.SetPos(80);								 // refresh progress bar
	m_doing = ("Recompressing " + ttdfile).c_str();			 // refresh state information
	UpdateData(FALSE);

	//////////////////////////////////////////////////////////
	// 3. TTD savegame compression
	//
	// TTD savegames are built up like TTO savegames, except
	// that the title is 47 bytes long and the saved data
	// may be up to about 600 KB long.
	//
	// The file checksum calculation is also done in this section.

	// prepare for TTD compression
	unsigned char *ttd_co = new unsigned char [618873]; // compressed data, size unknown, at most 618873
	const unsigned char *pttd_dec = ttd_dec; // pointer to decompressed data
	const unsigned char *const pttd_dec_end = ttd_dec + 618873; // pointer past the end of raw TTD data
	const unsigned char *ppos = pttd_dec;	 // pointer to the currently read byte
	unsigned long co_c = 0;					 // ttd_co writing counter

	// RLE compression (each iteration encodes one RLE sequence)
	while (pttd_dec < pttd_dec_end)
	{
		++ppos;
		// evaluate number of successive equal bytes
		while (ppos < pttd_dec_end && *pttd_dec == *ppos && ppos - pttd_dec <= 128)
			++ppos;
		if (ppos - pttd_dec >= 3) // at least 3 successive equal bytes?
		{
			ttd_co[co_c] = (unsigned char)(pttd_dec - ppos + 1); // write number of successive equal bytes
			++co_c;
			ttd_co[co_c] = *pttd_dec; // write the repeated byte
			++co_c;
			pttd_dec = ppos;
		}
		else
		{
			// evaluate number of incompressible bytes
			while (ppos < pttd_dec_end && (*ppos != *(ppos + 1) || *ppos != *(ppos + 2)) && ppos - pttd_dec < 128)
				++ppos;
			ttd_co[co_c] = (unsigned char)(ppos - pttd_dec - 1); // write it
			++co_c;
			// copy/write the incompressible bytes
			while (pttd_dec < ppos)
			{
				ttd_co[co_c] = *pttd_dec; // copy/write 1 byte
				++co_c;
				++pttd_dec;
			}
		}
	}

	// file checksum calculation
	unsigned long file_checksum = 0; // file checksum calculation begins with 0
	for(i = 0; i < 47; ++i)
	{
		// add ttd_title[i] to the lowest byte of the checksum
		file_checksum = (unsigned char)(file_checksum + ttd_title[i]) | (file_checksum & 0xFFFFFF00uL);
		// rotate checksum 3 bits to the left
		file_checksum = (file_checksum << 3) | (file_checksum >> 29);
	}
	for(i = 0; i < 2; ++i)
	{
		file_checksum = (unsigned char)(file_checksum + title_checksum[1-i]) | (file_checksum & 0xFFFFFF00uL);
		file_checksum = (file_checksum << 3) | (file_checksum >> 29);		
	}
	for(i = 0; i < co_c; ++i)
	{
		file_checksum = (unsigned char)(file_checksum + ttd_co[i]) | (file_checksum & 0xFFFFFF00uL);
		file_checksum = (file_checksum << 3) | (file_checksum >> 29);		
	}
	file_checksum += 201100;

	// write TTD savegame
	FILE * ttd = fopen((ttdpath + ttdfile).c_str(), "wb"); // open new file (write in binary mode)
	fwrite(ttd_title, 1, 47, ttd);						   // write savegame title
	fwrite(&title_checksum[1], 1, 1, ttd);				   // write title checksum shifted 8 bits...
	fwrite(&title_checksum[0], 1, 1, ttd);				   // ...because the title is 8 bytes longer
	fwrite(ttd_co, 1, co_c, ttd);						   // write compressed data
	fwrite(&file_checksum, 1, 4, ttd);					   // write file checksum
	fclose(ttd);

	// clean up heap
	delete [] ttd_co;
	delete [] ttd_dec;
	
	// TTD savegame compressed and written
	//////////////////////////////////////////////////////////

	m_progressline.SetPos(100); // refresh progress bar
	m_doing = ("Successfully converted " + ttofile + " to " + ttdfile).c_str(); // refresh state information
	UpdateData(FALSE);

}
